/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.server.core.globalConfigurations.GlobalConfigurationsSearchProvider");

dojo.require("jazz.ui._SearchProvider");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationResource");
var GlobalConfigurationResource = com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationResource;

dojo.requireLocalization("com.ibm.team.vvc.server.core.globalConfigurations", "GlobalConfigurationsSearchProviderMessages");
var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.globalConfigurations", "GlobalConfigurationsSearchProviderMessages");

var mutableGlobalConfigurationIcon = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/globalconfiguration_mutable.gif');
var immutableGlobalConfigurationIcon = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/globalconfiguration_immutable.gif');
var allGlobalConfigurationIcon = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/globalconfiguration_mutable.gif');

var mutableGlobalConfigurationsScope = "mutableGlobalConfigurations";
var immutableGlobalConfigurationsScope = "immutableGlobalConfigurations";
var allGlobalConfigurationsScope = "allGlobalConfigurations";

dojo.declare("com.ibm.team.vvc.server.core.globalConfigurations.GlobalConfigurationsSearchProvider", jazz.ui._SearchProvider, {
	
	scopes: [
		{ id: mutableGlobalConfigurationsScope, label: messages.mutableGlobalConfigurationsScopeTitle, icon: mutableGlobalConfigurationIcon},
		{ id: immutableGlobalConfigurationsScope, label: messages.immutableGlobalConfigurationsScopeTitle, icon: immutableGlobalConfigurationIcon},
		{ id: allGlobalConfigurationsScope, label: messages.allGlobalConfigurationsScopeTitle, icon: allGlobalConfigurationIcon},
	],
		
    tooltip: messages.tooltip,
    defaultText: messages.defaultText,
            
	onUpdate: function(args){
		this._getResults(args).then(dojo.hitch(this, function(response){
			var data = response.resultUris;
			var results = {
				results : [],
				paging: true,	// If you set this to false, the paginator will never update!
				totalResults: response.resultUris.length // resultUris MUST be returned
			};
			var deferreds = [];
			
			// Calculate the page to snow
			var start = (args.page - 1) * args.maxResults;
			var end = start + Math.min(args.maxResults, data.length - start);
			
			// Get only the results for the page to show
			for(var index = start; index < end; index++){
				var gcUrl = response.resultUris[index];
				var gcDataDeferred = jazz.client.xhrGet({
					url: gcUrl,
					preventCache: true,
					headers: {
						"Accept" : "text/json",
					}
				});
				
				deferreds.push(new dojo.Deferred());
				var resolutionIndex = 0;
				
				gcDataDeferred.then(dojo.hitch(this, function(gcJson){
					var gc = new GlobalConfigurationResource();
					gc.onLoad(dojo.fromJson(gcJson));
					results.results.push(this._createResult(gc, args));
					deferreds[resolutionIndex++].resolve(true);
				}), dojo.hitch(this, function(error){
					args.errback(messages.errorGettingResults);
				}));
			}
			
			// Wait for the deferreds to resolve, then show the results
			var deferredList = new dojo.DeferredList(deferreds);
			deferredList.then(dojo.hitch(this, args.callback, results));
		}), dojo.hitch(this, function(error){
			args.errback(messages.errorGettingResults);
		}));
	},
	
	_getResults: function(args){
		var queryParams = {
			query: args.query,
		};
		
		if(args.scope !== allGlobalConfigurationsScope){
			queryParams.mutable = args.scope === mutableGlobalConfigurationsScope;
		};
		
		var args = {
			url: VvcUtils.getVvcUri() +"/globalConfigurations/queryForTitle?" + dojo.objectToQuery(queryParams),
			headers: {
				"Accept" : "application/json",
				"Content-Type" : "application/json"
			},
			handleAs: "json",
			preventCache: true
		};
		return jazz.client.xhrGet(args);
	},
		
	_createResult: function(gc, args){
		return {
			uri: gc.getUri(),
			icon: gc.getMutable() ? mutableGlobalConfigurationIcon : immutableGlobalConfigurationIcon,
			label: this._formatResultLabel(gc.getTitle(), args.query)		
		}
	},
	
	_formatResultLabel: function(string, query){
		if(!string || string.length === 0){
			return "";
		}
		
		var start = string.toLocaleLowerCase().indexOf(query.toLocaleLowerCase());
		var end = start + query.length;
		
		if(start < 0){
			return string;
		}
		
		var label = string.substring(0, start);
		label += "<b>" + string.substring(start, end) + "</b>";
		label += this._formatResultLabel(string.substring(end), query);
		
		return label;
	}
});
})();
