/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
	dojo.provide("com.ibm.team.vvc.server.core.globalConfigurations.WebApplication");

	dojo.require("com.ibm.team.app.web.RepositoryApplication");
	dojo.require("net.jazz.ajax.ui.PlatformUI");
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	
	dojo.require("jazz.app.auth");
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");

	dojo.requireLocalization("com.ibm.team.vvc.server.core.globalConfigurations", "WebApplicationMessages");
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.globalConfigurations", "WebApplicationMessages");
	
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;

	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	var VVC_APP = "com.ibm.team.vvc.server.core";

	dojo.declare("com.ibm.team.vvc.server.core.globalConfigurations.WebApplication", com.ibm.team.app.web.RepositoryApplication, { 

		name: messages.globalConfigurations,
		
		iconSmall: dojo.moduleUrl(VVC_APP, "graphics/VVC_16.png"),
		iconLarge: dojo.moduleUrl(VVC_APP, "graphics/VVC_24.png"),
		autoRequestInitData: false,
		
		constructor: function(args) {
			this.config.navbarItems.push({
				label: messages.configurations,
				href: VvcUtils.getVvcUri() + "/web"
			});
			
			this.config.navbarItems.push({
				label: messages.globalConfigurations,
				href: VvcUtils.getVvcUri() + "/gc"
			});
			
			/*
			 * We will define the menu ourselves so that we can control the href
			 * to Configuration Explorer with a direct link without query parameters.
			 */
			this.config.usePageRegistry = false;
			
			// TMA: Hack to avoid cross-domain blocks. We call AdminClient.getUserRoles first, since
			// any TeamServerClient.invokeService calls will fail when not authenticated
			// See WI: https://jazz.net/jazz/resource/itemName/com.ibm.team.workitem.WorkItem/202484
			var srh = new ServiceResponseHandler(this, "_fetchUserRolesSuccess", "_fetchInitDataFailure");
			AdminClient.getUserRoles(srh);
		},
		
		_fetchUserRolesSuccess: function() {
			this.fetchInitData();
		},
		
		_fetchInitDataFailure: function() {
			this.inherited(arguments);
			this.ui.set("bannerTitle", this.getCurrentContext().name);
		},
		
		getCurrentContext: function() {
			return {name: messages.globalConfigurationManagement};
		},
		
		logout: function() {
			if (!net.jazz.ajax.ui.PlatformUI.getWorkbench().shouldVetoNavigation()) {
				var deferred = jazz.client.xhrPost({
					url: VvcUtils.getVvcUri() + "/repository/logout",
					sync: true,
					load: function(res, ioArgs){
						jazz.client.xhrPost({
							url: ioArgs.xhr.getResponseHeader("X-rmps-logout-url"),
							headers: {
								"X-ibm-rmps-internal": "true"
							},
							sync: true,
							load: function(res, ioArgs){
								/*Required procedure to prevent the o auth interceptor from refreshing the auth information before the log out
								* process is done
								* Block popups first before reloading the page
								* 
								*IE onload behavior is different on onload way. So, we have to do it like that way 
								*/
								var frame = dojo.doc.createElement(dojo.isIE ? '<iframe onload="window.frameLoadCallback()">' : 'iframe');
						        frame.onload = window.frameLoadCallback = function() {
						           	/*Now, refresh the page after making sure that the interceptor popups will not effect the log out taking place.*/
									dojo.doc.location.reload();
						        };
								/*We MUST refresh after making sure that the logout dialog is closed.
								 * So, we do that in the unload method of its window*/
								/*Now, refresh the page after making sure that the interceptor popups will not effect the log out taking place.*/
								frame.src= net.jazz.ajax._dataServerRoot + "/auth/logout";
								frame.style.display= "none";
								
								dojo.body().appendChild(frame);
							}
						});
					}
				});
				deferred.addCallback(this, function (info) {
					window.location.reload(true);
				});
				deferred.addErrback(function(error) {
					var messageBox = new jazz.ui.MessageBox();
					messageBox.showMessage(error);
				});
			}
			return false;
		},

		uninitialize: function() {
			if (this._helpBox) this._helpBox.destroy();   
			if (this._helpMenu) this._helpMenu.destroy();
			if (this._navMenu) this._navMenu.destroy();
		}

	});
})();
