<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE mapper
    PUBLIC "-//mybatis.org//DTD Mapper 3.0//EN"
    "http://mybatis.org/dtd/mybatis-3-mapper.dtd">

<mapper namespace="Queries">

  <select id="dimensionQuery" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT target as dimension
    FROM VVC_ConfSpace_dimensions 
    WHERE source = #{confSpace}
  </select>
  
  <select id="dimensionOrderQuery" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_Dimension.uri as greater, oneGreaterThan as lesser
    FROM VVC_Dimension 
    JOIN VVC_ConfSpace_dimensions ON VVC_Dimension.uri = VVC_ConfSpace_dimensions.target 
    WHERE VVC_ConfSpace_dimensions.source = #{confSpace} and oneGreaterThan IS NOT NULL
  </select>
  
  <select id="enumerationValueOrderQuery" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_EnumerationValue.uri as greater, VVC_EnumerationValue.oneGreaterThan as lesser, VVC_Dimension.uri as dimension
    FROM VVC_EnumerationValue 
    JOIN VVC_Dimension ON VVC_EnumerationValue.dimension = VVC_Dimension.uri 
    JOIN VVC_ConfSpace_dimensions ON VVC_Dimension.uri = VVC_ConfSpace_dimensions.target 
    WHERE VVC_ConfSpace_dimensions.source = #{confSpace}
  </select>
  
  <select id="maxValueOfNextLesserDimensionQuery" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT maxEnumValueOfNextLesserDim, maxIntValueOfNextLesserDim, VVC_EnumerationValue.uri as value
    FROM VVC_EnumerationValue 
    JOIN VVC_Dimension ON VVC_EnumerationValue.dimension = VVC_Dimension.uri 
    JOIN VVC_ConfSpace_dimensions ON VVC_Dimension.uri = VVC_ConfSpace_dimensions.target 
    WHERE VVC_ConfSpace_dimensions.source = #{confSpace} and (maxEnumValueOfNextLesserDim IS NOT NULL or maxIntValueOfNextLesserDim IS NOT NULL)
  </select>
  
  <select id="conceptUrisQuery" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT concept
    FROM VVC_Mapping 
    JOIN VVC_Point ON VVC_Point.uri = VVC_Mapping.point 
    WHERE VVC_Point.configurationSpace = #{confSpace}
  </select>
  
  <select id="mappingQuery" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_Mapping.uri as mapping, VVC_Point.uri as point, timeOrder, purpose, changeSet, pkey as dimension, svalue as enum_value, ivalue as int_value
    FROM VVC_Point 
    JOIN VVC_Point_properties ON VVC_Point_properties.uri = VVC_Point.uri 
    JOIN VVC_Mapping ON VVC_Point.uri = VVC_Mapping.point 
    WHERE VVC_Mapping.concept = #{concept} AND VVC_Point.configurationSpace = #{confSpace}
  </select>
  
  <select id="mappingBulkQuery" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_Mapping.concept, VVC_Mapping.uri as mapping, VVC_Point.uri as point, timeOrder, purpose, changeSet, pkey as dimension, svalue as enum_value, ivalue as int_value
    FROM VVC_Point 
    JOIN VVC_Point_properties ON VVC_Point_properties.uri = VVC_Point.uri 
    JOIN VVC_Mapping ON VVC_Point.uri = VVC_Mapping.point 
    WHERE VVC_Mapping.concept IN
      <foreach item="concept" index="index" collection="concepts"
        open="(" separator="," close=")">
          #{concept}
      </foreach>
      AND VVC_Point.configurationSpace = #{confSpace}
  </select>
    
  <select id="simpleMappingQuery" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_Mapping.uri as mapping
    FROM VVC_Mapping 
    JOIN VVC_Point ON VVC_Point.uri = VVC_Mapping.point 
    WHERE VVC_Mapping.concept = #{concept} AND VVC_Point.configurationSpace = #{confSpace} 
  </select>
  
  <select id="versionQuery" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_Mapping.uri as mapping, VVC_Mapping.version
    FROM VVC_Mapping 
    JOIN VVC_Point ON VVC_Point.uri = VVC_Mapping.point 
    WHERE VVC_Mapping.concept = #{concept} and VVC_Point.uri = #{point}  
  </select>

  <select id="mappingAndVersionBulkQuery" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_Mapping.concept, VVC_Mapping.point, VVC_Point.timeOrder, VVC_Mapping.uri as mapping, VVC_Mapping.version,
                    VVC_System.creator, VVC_System.created
    FROM VVC_Mapping 
    JOIN VVC_Point ON VVC_Point.uri = VVC_Mapping.point 
    JOIN VVC_System ON VVC_System.id = VVC_Mapping.uri 
    WHERE VVC_Mapping.concept IN
      <foreach item="concept" index="index" collection="concepts"
        open="(" separator="," close=")">
          #{concept}
      </foreach>
      AND VVC_Point.uri IN
      <foreach item="point" index="index" collection="points"
        open="(" separator="," close=")">
          #{point}
      </foreach>
  </select>

  <select id="mappingAndVersionBulkQueryWithChangesets" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_Mapping.concept, VVC_Mapping.point, VVC_Point.timeOrder, VVC_Mapping.uri as mapping, VVC_Mapping.version,
                    VVC_System.creator, VVC_System.created,
                    VVC_ChangeSet.uri as changeset, VVC_ChangeSet.title
    FROM VVC_Mapping 
    JOIN VVC_Point ON VVC_Point.uri = VVC_Mapping.point 
    JOIN VVC_System ON VVC_System.id = VVC_Mapping.uri 
    LEFT JOIN VVC_ChangeSet ON VVC_ChangeSet.endPoint = VVC_Point.uri 
    WHERE VVC_Mapping.concept IN
      <foreach item="concept" index="index" collection="concepts"
        open="(" separator="," close=")">
          #{concept}
      </foreach>
      AND VVC_Point.uri IN
      <foreach item="point" index="index" collection="points"
        open="(" separator="," close=")">
          #{point}
      </foreach>
  </select>
  
  <select id="findConfigurationsOtherThanSnapshotsQuery" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_ConfResource.uri as configuration, VVC_Point.uri as point, base, title
    FROM VVC_ConfResource 
    JOIN VVC_Point ON VVC_Point.uri = VVC_ConfResource.point 
    WHERE VVC_Point.configurationSpace = #{space} AND VVC_Point.timeOrder IS NULL
  </select>
  
  <select id="unhandledMappings" parameterType="long" resultType="hashmap">
    SELECT DISTINCT VVC_Mapping.uri as mapping, created
    FROM VVC_Mapping
    JOIN VVC_System ON VVC_System.id = VVC_Mapping.uri
    WHERE created > #{timestamp}
  </select>
  
  <select id="CompareProviderStorageService_getStorageUriFromProviderUri" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT uri
    FROM VVC_CompareProvidersStorage
    WHERE providerServiceUri = #{providerUri}
  </select>
  
  <select id="findConfigurations" resultType="hashmap">
    SELECT DISTINCT VVC_ConfResource.uri as conf, mutable, archived, securityContext, VVC_ConfResource.title, creator, created, controlledBy, VVC_Point.uri as point, VVC_Point.configurationSpace as confSpace, VVC_ConfResource.description, base   
    FROM VVC_ConfResource
    JOIN VVC_Point ON VVC_Point.uri = VVC_ConfResource.point 
    JOIN VVC_System ON VVC_System.id = VVC_ConfResource.uri 
  </select>
  
  <select id="allSpacesQuery" resultType="hashmap">
    SELECT DISTINCT uri, title, description   
    FROM VVC_ConfSpace
  </select>
  
  <select id="findSetting" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT uri   
    FROM VVC_Setting
    WHERE id = #{settingId}
  </select>
  
  <select id="getHighestOrderedDimensionsForConfigurationSpace" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT result.uri as dimensionUri   
    FROM VVC_Dimension result
    JOIN VVC_ConfSpace_dimensions ON result.uri = VVC_ConfSpace_dimensions.target 
    WHERE VVC_ConfSpace_dimensions.source = #{configurationSpaceUri} AND NOT EXISTS (
      SELECT 1
      FROM VVC_Dimension dim
      WHERE dim.oneGreaterThan = result.uri)
  </select>
  
  <select id="getEnumerationValuesForDimension" parameterType="hashmap" resultType="EnumerationValue">
    SELECT * FROM VVC_EnumerationValue 
    JOIN VVC_System ON VVC_EnumerationValue.uri = VVC_System.id 
    WHERE dimension = #{dimension}
  </select>
  
  <select id="getTimeOrder" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT timeOrder as value
    FROM VVC_Point 
    WHERE uri = #{point}
  </select>
  
  <select id="getTimeOrdersBulk" parameterType="hashmap" resultType="hashmap">
    SELECT uri, timeOrder as value
    FROM VVC_Point 
    WHERE uri IN
      <foreach item="point" index="index" collection="points"
               open="(" separator="," close=")">
          #{point}
      </foreach>
  </select>
  
  <select id="getAllCompareProvidersStorage" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT uri as compareProvidersStorage
    FROM VVC_CompareProvidersStorage 
  </select>
  
  <select id="getAllPointsInSpace" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT uri
    FROM VVC_Point
    WHERE configurationSpace = #{space} 
  </select>
  
  <select id="getAllMappingsInSpace" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_Mapping.uri
    FROM VVC_Mapping
    JOIN VVC_Point ON VVC_Point.uri = VVC_Mapping.point 
    WHERE configurationSpace = #{space} 
  </select>
  
  <select id="getAllDependencyResourcesInSpace" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT version as uri
    FROM VVC_Mapping
    WHERE concept = #{concept} 
  </select>
  
  <select id="getAllChangeSetsInSpace" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_ChangeSet.uri
    FROM VVC_ChangeSet
    JOIN VVC_Point ON VVC_Point.uri = VVC_ChangeSet.endPoint 
    WHERE configurationSpace = #{space} 
  </select>
  
  <select id="getAllConfigurationsInSpace" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_ConfResource.uri
    FROM VVC_ConfResource
    JOIN VVC_Point ON VVC_Point.uri = VVC_ConfResource.point 
    WHERE configurationSpace = #{space} 
  </select>
  
  <select id="getAllDimensions" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT uri
    FROM VVC_Dimension 
  </select>
  
  <select id="getConfigurationSpaceForProjectArea" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_ConfSpace.uri as configurationSpace
    FROM VVC_ConfSpace
    JOIN VVC_System ON VVC_System.id = VVC_ConfSpace.uri
    WHERE securityContext = #{projectAreaUri} 
  </select>
  
  <select id="getSecurityContext" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT securityContext
    FROM VVC_System 
    WHERE id = #{uri} 
  </select>
  
  <select id="getGlobalConfigurations" resultType="hashmap">
    select DISTINCT uri from VVC_GlobalConfiguration 
  </select>

  <select id="getGlobalConfigurationsWithContributions" resultType="hashmap">
    select source, target from VVC_GC_contributions
  </select>
  
  <select id="getGlobalConfigurationsWithTitle" parameterType="hashmap" resultType="hashmap">
    select DISTINCT uri from VVC_GlobalConfiguration
    WHERE lower(title) like #{text}  
  </select>
  
  <select id="getGlobalConfigurationsWithTitleAndMutable" parameterType="hashmap" resultType="hashmap">
    select DISTINCT uri from VVC_GlobalConfiguration
    WHERE lower(title) like #{text} AND mutable = #{isMutable}
  </select>
  
</mapper>
