/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){   
	
	dojo.provide("com.ibm.team.vvc.server.core.ui.configurations.steps.AddToExisting");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dojo.date.stamp");
	dojo.require("jazz.ui.wizard.WizardStep");
	dojo.require("dijit.form.Select");
	dojo.require("dojox.validate.web");
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationResource");
		
	dojo.require('dojo.i18n');
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.configurations.steps", "AddToExistingMessages");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	dojo.require("com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker");
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.configurations.steps.AddToExisting", [jazz.ui.wizard.WizardStep, dijit._Widget, dijit._Templated], {
	
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/configurations/steps/templates/AddToExisting.html'),
		messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.configurations.steps", "AddToExistingMessages"),
		widgetsInTemplate: true,
		_storageKey: null,
		_data: null,
		
		parentConfigurationPicker: null,
	
		_disconnects: null,
		_destroys: null,
		
		_sizeOpen: {	
			width: 600,
			height: 270
		},
			
		_sizePickerDimensionsOpen: {
			width: 600,
			height: 600
		},
			
		_disabed: null,
			
		constructor: function(args){
			if(args && args.storageKey){
				this._storageKey = args.storageKey;
			}
			this._disconnects = [];
			this._destroys = [];
		},
		
		destroy: function(args){
			this.inherited(args);
			dojo.forEach(this._disconnects, function(handle){dojo.disconnect(handle);});
			dojo.forEach(this._destroys, function(obj){dojo.destroyRecursive(obj);});
		},
		
		postCreate: function(){
			this._data = dojo.fromJson(sessionStorage.getItem(this._storageKey));
			
			if (!this._data.input.configuration) {
				this.stepTitle = this.messages.newTitle;
			} else {
				this.stepTitle = this.messages.addTitle;
			}
			
			var events = ["keyup", "onpaste", "oncut"];
			for (var i = 0; i < events.length; i++) {
				this._disconnects.push(dojo.connect(this.configurationNameInput, events[i], dojo.hitch(this, this.onNameInputChange)));
			}			
		},
		
		focusStep: function(){
			this._data = dojo.fromJson(sessionStorage.getItem(this._storageKey));

			//in case we are accessing this page again
			this._data.addToExistingGlobalConfiguration = null;
			
			if (this._data.addToWhat === "existing") {
				this.showBasePicker();
			} else if (this._isAddToNewOrMakeNewConfig()) {
				this.showNewConfigDetails();
			}
			
			//if neither one, a big error occurred so we cannot really show anything
			
			this.updateSize();
		},
		
		showNewConfigDetails: function() {
			dojo.removeClass(this.newConfigurationNameRow, "hidden");
			dojo.removeClass(this.newConfigurationDescriptionRow, "hidden");
			
			dojo.addClass(this.parentConfigurationPickerNode, "hidden");
		},
			
		customOnFinish: function() {
			if (this._isAddToNewOrMakeNewConfig()) {
				var newGlobalConfiguration = {
					title: this.configurationNameInput.value.trim(),
					description: this.configurationDescriptionInput.value.trim(),
					mutable: true
				};
				
				this._data.addToNewGlobalConfiguration = newGlobalConfiguration;
				sessionStorage.setItem(this._storageKey, dojo.toJson(this._data));
			}
		},
		
		canFlipToNextPage: function(){
			if (this._data.addToWhat === "existing") {
				return this._data.addToExistingGlobalConfiguration;
			} else if (this._isAddToNewOrMakeNewConfig()) {
				return this.configurationNameInput.value.trim().length > 0;
			}
			
		},
		
		_isAddToNewOrMakeNewConfig: function() {
			return this._data.addToWhat === "new" || !this._data.input.configuration;
		},
		
		onNameInputChange: function() {
			this.triggerButtonStateUpdate();
		},
		
		canFinish: function() {
			return this.canFlipToNextPage();
		},
				
		togglePicker: function(){
			this.showBasePicker();
			this.updateSize();
		},
		
		showBasePicker: function(){
			// Lazily instantiate the picker
			if(!this.parentConfigurationPicker){
				var validParentTypes = this._data.input.validParentTypes;
				this.parentConfigurationPicker = new com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker(
					{
//						validTypes : this._data.newConfiguration.type === "workspace" ? "snapshot" : "workspace",
						mutableGlobalConfigurationsOnly: true,
						closeable : false
					});
				this.parentConfigurationPicker.hideOkCancelButtons();

				this._disconnects.push(dojo.connect(this.parentConfigurationPicker, "onOk", dojo.hitch(this, this.changeParentConfiguration)));
				this._disconnects.push(dojo.connect(this.parentConfigurationPicker, "onSelect", dojo.hitch(this, this.changeParentConfiguration)));
				
				this.parentConfigurationPickerNode.appendChild(this.parentConfigurationPicker.domNode);
			}
			dojo.addClass(this.newConfigurationNameRow, "hidden");
			dojo.addClass(this.newConfigurationDescriptionRow, "hidden");
			
			dojo.removeClass(this.parentConfigurationPickerNode, "hidden");
		},
		
		updateSize: function(){
			if(this._data.addToWhat === "existing"){
				this.setSize(this._sizePickerDimensionsOpen);
			} else /*if (this._data.addToWhat === "new")*/ {
				this.setSize(this._sizeOpen);
			}
		},
		
		changeParentConfiguration: function(newConfiguration){
			this._data.addToExistingGlobalConfiguration = newConfiguration;
			sessionStorage.setItem(this._storageKey, dojo.toJson(this._data));
			this.triggerButtonStateUpdate();
		},
		
		_setConfigurationNameInputFocus: function() {
			//TODO
		},
		
		_setConfigurationDescriptionInputFocus: function() {
			//TODO
		}
				
	});

})();