/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){   
	
	dojo.provide("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationProgressPage");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dijit.form.RadioButton");
	dojo.require("jazz.ui.wizard.WizardStep");
	dojo.require("jazz.app.ua.PageHoverHelp");
	
	dojo.require('dojo.i18n');
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.configurations.steps", "ConfigurationProgressPageMessages");
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");		
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
	var NotificationFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationProgressPage", [jazz.ui.wizard.WizardStep, dijit._Widget, dijit._Templated], {
	
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/configurations/steps/templates/ConfigurationProgressPage.html'),
		messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.configurations.steps", "ConfigurationProgressPageMessages"),
		widgetsInTemplate: true,
		_storageKey: null,
		_data: null,
		_sizeExternal: {
			width: 600, 
			height: 280
		},
		_sizeInternal: {
			width: 600,
			height: 225 
		},
		_steps: null,
		_isComplete: null,
		_isStarted: null,
		_requiredCount: null,
		_completeCount: null,
		parentConfiguration: null,
		failed: false,
		
		_disconnects: null,
		_destroys: null,
		
		constructor: function(args){
			if(args && args.storageKey){
				this._storageKey = args.storageKey;
			}
			
			this._isComplete = false;
			this._steps = [];
			this._disconnects = [];
			this._destroys = [];
			
			// IE8 is doing something weird with sizes...
			if(dojo.isIE <= 8){
				this._sizeExternal.height = 260;
			}
		},
		
		destroy: function(args){
			this.inherited(args);
			dojo.forEach(this._disconnects, function(handle){dojo.disconnect(handle);});
			dojo.forEach(this._destroys, function(obj){dojo.destroyRecursive(obj);});
		},
		
		postCreate: function(){
			this.stepTitle = this.messages.title;		
			this._disconnects.push(dojo.connect(this.nextActionRadio, "onclick", dojo.hitch(this, this.triggerButtonStateUpdate)));
		},
		
		focusStep: function(){
			/* 
			 * +--- Workaround for foundation defect 247951 ---+
			 * 
			 * This will mess up the UI somewhat, but guarantees that a second attempt 
			 * to create the configuration is not submitted to the server.
			 */
			if(this._isStarted){
				console.warn("Preventing second attempt to create a configuration!")
				return;
			}
			this._isStarted = true;
			/* +----------------------------------------------+ */

			
			this._data = dojo.fromJson(sessionStorage.getItem(this._storageKey));
			this.updateSize();
						
			this._createConfigurationStep(this._data.newConfiguration);
			
			// Only wait for the configuration we care about.
			this._createPreparingStep(this._data.newConfiguration);
			
			setTimeout(dojo.hitch(this, this.start), 100);
			
			this.returnToPreviousLabel.appendChild(dojo.doc.createTextNode(dojo.string.substitute(this.messages.returnToPrevious, {ncType: this._data.newConfiguration.typeLabel})));
			this.waitForConfigurationLabel.appendChild(dojo.doc.createTextNode(dojo.string.substitute(this.messages.waitForConfiguration, {ncType: this._data.newConfiguration.typeLabel})));
			
			if(this._data.vvcInternal){
				dojo.empty(this.nextActionRadioNode);
			} else {
				if(this._data.newConfiguration.type === "workspace"){
					this.nextActionRadio[0].checked = true;
					this.nextActionRadio[1].checked = false;
				} else if(this._data.newConfiguration.type === "snapshot"){
					this.nextActionRadio[0].checked = false;
					this.nextActionRadio[1].checked = true;
				}
			}
			
			var uaHelp = new jazz.app.ua.PageHoverHelp({uaKey: "com.ibm.team.vvc.server.core.wizard_ConfigurationProgressPage001"});
			this.returnToPreviousLabel.appendChild(uaHelp.domNode);
			this._destroys.push(uaHelp);
			
			this.triggerButtonStateUpdate();
		},
		
		_handleFailed: function(step) {
			if(!this._data.vvcInternal){
				dojo.query("input", this.nextActionRadio).attr("disabled", true);
			}
			dojo.addClass(this.returnToPreviousLabel, "nextActionsDisabled");
			dojo.addClass(this.waitForConfigurationLabel, "nextActionsDisabled");
			dojo.addClass(this.nextActionsHeading, "nextActionsDisabled");
			dojo.removeClass(step.errorInformationNode, "noError");
			this._disconnects.push(dojo.connect(step.errorInformationRetry, "onclick", dojo.hitch(this, this._selectWizardFirstPage)));
			this.failed = true;
			this.triggerButtonStateUpdate();
		},

		
		/**
		 * Creates the configuration step and adds it to the step list.
		 */
		_createConfigurationStep: function(configuration, retryAutomatically){
			
			var parent = configuration.parent;
			var space = configuration.space;
			var step;
			
			if(!space){
				// Something is very wrong, at least the space is required!
				return;
			}
			
			if(parent && !parent.uri) {
				// Ensure parent gets created first
				this._createConfigurationStep(parent, true);
			} 
			
			var theContext = {
				step: step,
				configuration: configuration,
				self: this,
				retryAutomatically: retryAutomatically
			};
			
			var onInProgressFunc = dojo.hitch(theContext, function(){
				var deferred = this.self._createConfiguration(configuration, this.retryAutomatically);
				dojo.when(deferred, dojo.hitch(this, function(params){
					this.configuration.uri = params;
				}), dojo.hitch(this, function(err) {
					this.step.setFailed();
					var hitched = dojo.hitch(this.self, this.self._handleFailed);
					hitched(this.step);
					
					var errorDetails = com.ibm.team.vvc.shared.core.ui.utils.ErrorUtil.getErrorDetails(err);
					if (errorDetails.errorStatus === 403) {
						this.step.showRetryMessage(this.self.messages.forbiddenAction);
					} else if (errorDetails.errorSubStatus === "EXISTS") {
						this.step.showRetryMessage(this.self.messages.configurationExistsAction);
					} else if (errorDetails.errorSubStatus === "PARENT_ARCHIVED") {
						this.step.showRetryMessage("");
					}
				}));
			});
						
			var args = {
					title: dojo.string.substitute(this.messages.createConfiguration, {ncType: configuration.typeLabel, ncTitle: jazz.util.html.escape(configuration.title)}),
					description: "",
					onInProgress: onInProgressFunc
			};
			
			step = new com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationProgressPage._step(args);
			theContext.step = step;
			this._addStep(step);
		},
		
		_selectWizardFirstPage: function(event) {
			this.wizard.selectStep(1);
			dojo.stopEvent(event);
		},
		
		_createPreparingStep: function(configuration){
			var step;
			
			if(this._requiredCount){
				this._requiredCount++;
			} else {
				this._requiredCount = 1;
				this._completeCount = 0;
			}
			
			var context = this;
			var onInProgressFunc = dojo.hitch(this, function(){
				var deferred = PageUtil.queryForConcept("http://jazz.net", configuration.uri);
				this._disconnects.push(dojo.connect(this, "customOnFinish", dojo.hitch(this, function(){
					deferred.cancel("", false);
				})));
				return deferred;
			});
			
			var postInProgressFunc = dojo.hitch(this, function(){
				this._completeCount++;
				this.triggerButtonStateUpdate();
			});
			
			var args = {
					title: dojo.string.substitute(this.messages.preparingConfiguration, {ncTitle: jazz.util.html.escape(configuration.title), ncType: configuration.typeLabel}),
					description: dojo.string.substitute(this.messages.preparingConfigurationDetails, {ncTitle: jazz.util.html.escape(configuration.title), ncType: configuration.typeLabel}),
					onInProgress: onInProgressFunc,
					postInProgress: postInProgressFunc,
					onFail: dojo.hitch(this, this._handleFailed)
			}
			step = com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationProgressPage._step(args);
			this._addStep(step);
		},
		
		//if retryAutomatically, that means we want a unique name and the name
		//was specified by the user
		//if !retryAutomatically, we still want a unique name, but the name was
		//not specified by the user (so we should automatically attempt to
		//generate a unique name) 
		_createConfiguration: function(configuration, retryAutomatically){	
			var parent = configuration.parent;
			var title = configuration.title;
			var description = configuration.description;
			var spaceUri = configuration.space.uri;
			var parentUri = parent && parent.uri ? parent.uri : null /*spaceUri*/; 
			var deferred = null;
			if(configuration.type === "workspace"){
				deferred = PageUtil.createWorkspace(title, description, parentUri, spaceUri, true, retryAutomatically);
			} else if(configuration.type === "snapshot") {
				deferred = PageUtil.createSnapshot(title, description, parentUri, spaceUri, true, retryAutomatically);
			}
			return deferred;
		},
		
		start: function(){
			if(this._steps[0]){
				this._start(0);
			}
		},
		
		// Start at the specified step
		_start: function(/*optional*/ stepNum){
			if(!stepNum){
				stepNum = 0;
			}
			
			if(stepNum >= this._steps.length){
				return;
			}
			
			this._steps[stepNum].start();
			if (!this._steps[stepNum].failed) {
				this._start(++stepNum);
			}
		},
						
		updateSize: function(){
			this.setSize(this._data.vvcInternal ? this._sizeInternal : this._sizeExternal);
		},
				
		canFinish: function(){
			if (this.failed) {
				return false;
			}
			
			if(this._data.vvcInternal){
				return this._requiredCount === this._completeCount;
			} // else
			
			if(this.nextActionRadio[0].checked && this._isComplete || this._requiredCount === this._completeCount){
				// nextActionRaido[0] = wait for the configuration to be ready
				return true;
			} else if(this.nextActionRadio[1].checked){
				// return to the previous configuration
				return true;
			}

			return false; 
		},
		
		canCancel: function(){
			return this.failed;
		},
		
		canFlipToPreviousPage: function(){
			return false;
		},
		
		_addStep: function(/*ConfigurationProgressPage._step*/ step){
			this._steps.push(step);
			this._destroys.push(step);
			this.progressList.appendChild(step.domNode);
		},
		
		customOnFinish : function(){
			this._data.actions = {};
			
			if(this._data.vvcInternal || this.nextActionRadio[1].checked){
				// Pass the newly created configuration back, but do not enter it.
				this._data.actions.add = this._data.newConfiguration;
			} else {
				// Pass the newly created configuration back, and enter it.
				this._data.actions.load = this._data.newConfiguration
			}

			sessionStorage.setItem(this._storageKey, dojo.toJson(this._data));
		}
				
	});
	
	/**
	 * Creates a step in for the progress page.  Steps have a 4-part lifecycle: 
	 * <ol>
	 * <li>WAIT: step is waiting to start</li>
	 * <li>IN PROGRESS: step is executing</li>
	 * <li>COMPLETE: step has completed successfully</li>
	 * <li>ERROR: step has encountered a problem</li>
	 * </ol>
	 * Each state has hooks for 'pre', 'on', and 'post' allowing a developer complete control over the lifecycle.  Implementations of each of
	 * the states are responsible for calling onError if something goes wrong, although the steps will manage their lifecycle independently.
	 */
	dojo.declare("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationProgressPage._step", [dijit._Widget, dijit._Templated], {
		
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/configurations/steps/templates/ConfigurationProgressPage._step.html'),
		messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.configurations.steps", "ConfigurationProgressPageMessages"),
		
		title: null,
		description: null,
		deferred: null,
		failed: false,
		//onFail: null, from mixin
		_state: null,
		_states: {
				wait: "WAIT",
				inprogress: "IN PROGRESS",
				complete: "COMPLETE",
				error: "ERROR"
		},
		
		_hooks: ["preWait", "onWait", "postWait", 
		         "preInProgress", "onInProgress", "postInProgress",
		         "preComplete", "onComplete", "postComplete"],

		_destroys: null,
		_disconnects: null,
		         
		constructor: function(args){
			dojo.safeMixin(this, args);
			this._state = this._states.wait;
			this.description = args.description;
			//this.onFail = args.onFail; from mixin
			this._destroys = [];
			this._disconnects = [];
		},
		
		destroy: function(args){
			this.inherited(args);
			dojo.forEach(this._disconnects, function(handle){dojo.disconnect(handle);});
			dojo.forEach(this._destroys, function(obj){dojo.destroyRecursive(obj);});
		},
		
		postCreate: function(){
			this.titleNode.innerHTML = this.title;
		},
		
		showRetryMessage: function(errorRetryHTML) {
			this.errorInformationRetry.innerHTML = errorRetryHTML;
		},
		
		start: function(){
			this._executeHooks(this._hooks);
		},
		
		setFailed: function(){
			this.failed = true;
			this._state = this._states.error;
			this._failed();
			if (this.onFail) {
				this.onFail(this);
			}
		},

		_executeHooks: function(/* Array of hooks names */ hooks, /*optional*/ index){
			if (this.failed) {
				return;
			}
			
			if(!index){
				index = 0;
			}
			
			if(index === 0){
				this._preExecute();
			}
			
			if(index > hooks.length){
				this._postExecute();
				return;
			}
			
			var curr = hooks[index];
			if(this[curr]){
				// Hooks can return null or a deferred
				var def = this[curr]();
				if(def){
					dojo.when(def, dojo.hitch(this, function(){
						this._executeHooks(hooks, ++index);
						if (this.failed) {
							return;
						}
					}), dojo.hitch(this, function(err) {
						this.setFailed();
					}));
				} else {
					this._executeHooks(hooks, ++index);
					if (this.failed) {
						return;
					}
				}
			} else {
				this._executeHooks(hooks, ++index);
				if (this.failed) {
					return;
				}
			}
		},
		
		// Private handlers
		_preExecute: function(){
			// Set the icon, show the "show more"
			dojo.addClass(this.domNode, "inProgress");
			dojo.removeClass(this.domNode, "waiting");
		},
		
		_postExecute: function(){
			// Set the icon, hide the "show more"
			dojo.addClass(this.domNode, "complete");
			dojo.removeClass(this.domNode, "inProgress");
		},
		
		_failed: function(){
			// Set the icon, show the "show more"
			dojo.removeClass(this.domNode, "inProgress");
			dojo.addClass(this.domNode, "error");
		},		
		
		// Wait hooks
		preWait: null,
		onWait: null,
		postWait: null,
		
		// In Progress hooks
		preInProgress: null,
		onInProgress: null,
		postInProgress: null,
		
		// Complete hooks
		preComplete: null,
		onComplete: null,
		postComplete: null,
		_postComplete: null,
	});
})();