/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.server.core.ui.explorer.VvcExplorer");

dojo.require('dojo.i18n');
dojo.require("dijit._Widget");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.toolbar.Separator");

dojo.require("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGrid");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcCache");

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;

dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory");
var buttonFactory = com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.server.core.ui.explorer.VvcExplorerTable");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.explorer", "VvcExplorerMessages");

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;

dojo.declare("com.ibm.team.vvc.server.core.ui.explorer.VvcExplorer", [com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGrid], {
	  
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.explorer", "VvcExplorerMessages"),
	_currentView: null,
	_viewStructureCache: null,
	_configurationCache: null,
	_showArchived: null,
	_actionsBar: null,
		
	constructor: function __VvcExplorer(args){
		this._showArchived = false;
	},
		
	_initializeDatastore: function __initializeDatastore(args){
		if(this._configurationCache){
			this._configurationCache.clear();
		} else {
			this._configurationCache = new com.ibm.team.vvc.shared.web.ui.util.VvcCache({
				storageKey: vvcConstants.CONFIGURATION_CACHE_KEY
			});
		}
		
		if(this._viewStructureCache){
			this._viewStructureCache.clear();
		} else {
			this._viewStructureCache = new com.ibm.team.vvc.shared.web.ui.util.VvcCache({
				target: VvcUtils.getVvcUri() + "/explore/configurations",
				bulkTarget: VvcUtils.getVvcUri() + "/explore/configurations",
				storageKey: "viewCache"
			});	
		}
	},
		
	buildTitleNode: function __buildTitleNode(){
		var container = dojo.create("span");
		var iconNode = dojo.create("span", {
			innerHTML: "&nbsp;",
			"class": "icons-explorer-configuration"
		});
		var titleNode = dojo.create("span", {
			innerHTML: this.messages.configurationsPageTitle,
			"class": "explorerTableTitle"
		});
		var titleUa = new jazz.app.ua.PageHoverHelp({uaKey: "com.ibm.team.vvc.server.core.page_ConfigurationsExplorer001"});
		
		container.appendChild(iconNode);
		container.appendChild(titleNode);
		container.appendChild(titleUa.domNode);
		return container;
	},
	

	buildActionsNode: function __buildActionsNode(){
		var actionsBar = new jazz.ui.Toolbar();
		this._actionsBar = actionsBar;
		
//TODO: This will be used for setting master VVC
//		actionsBar.addChild(new jazz.ui.toolbar.Separator());
//		actionsBar.addChild(buttonFactory.getTemporarySetMasterVVCButton());		
//		actionsBar.addChild(new jazz.ui.toolbar.Separator());
		
		var compareButton = buttonFactory.getCompareButton(null,{disabled: true, title: this.messages.compareButtonTitle},dojo.hitch(this, function(){
			var length = this._selectedRows.order.length;
			var args = {
				sourceUri: this._selectedRows.order[0],
				targetUri: length === 2 ? this._selectedRows.order[1] : null,
			};
			return args;
		}));
		actionsBar.addChild(compareButton);
		
		var deliverButton = buttonFactory.getDeliverButton(null,{disabled: true},dojo.hitch(this, function(){
			var length = this._selectedRows.order.length;
			var args = {
				sourceUri: this._selectedRows.order[0],
				targetUri: length === 2 ? this._selectedRows.order[1] : null,
			};
			return args;
		}));
		actionsBar.addChild(deliverButton);		
		actionsBar.addChild(new jazz.ui.toolbar.Separator());
		actionsBar.addChild(buttonFactory.getCollapseAllButton(null,{actionCallback: dojo.hitch(this, this.collapseAll)}));
		actionsBar.addChild(buttonFactory.getRefreshButton(null,{actionCallback: dojo.hitch(this, this.refresh, true)}));
		actionsBar.addChild(new jazz.ui.toolbar.Separator());
		var showArchivedButton = buttonFactory.getShowArchivedButton(null,{actionCallback: dojo.hitch(this, function(){
			this.showArchived(showArchivedButton.pressed);
		})}) 
		actionsBar.addChild(showArchivedButton);
//		actionsBar.addChild(this._getViewpointsButton());
		
		dojo.connect(this, "onRowSelectionChanged", dojo.hitch(this, function __handleOnRowSelectionChanged(){
			var length = this._selectedRows.order.length;
			var order = this._selectedRows.order;
			var data = this._selectedRows.data;
			
			// Need at least one selection
			if(length < 2){
				compareButton.set("enabled", false);
				deliverButton.set("enabled", false);
				return;
			}			
			
			// Deliver and compare need 1-2 selections 
			if(length >= 1 && length <= 2){
				var spaceUri = data[order[0]].spaceUri;
				
				// compare button...
				var enabled = true;
				for(var index = 0; index < length; index++){
					// Nobody is archived
					enabled = enabled && !data[order[index]].archive;
					// Everyone must belong to the same configuration space
					enabled = enabled && data[order[index]].spaceUri == spaceUri;
				}
				compareButton.set("enabled", enabled);

				/*
				 * Enables/disables the deliver button based on the criteria that at least one of
				 * the selected items must be a workspace. 
				 */
				var enabled = false;
				if (order[0] && order[1] && 
						(data[order[0]].type === "workspace" || data[order[1]].type === "workspace")) {
					enabled = true;
				}
				for(var index = 0; index < length; index++){
					// Nobody is archived
					enabled = enabled && !data[order[index]].archive;
					// Everyone must belong to the same configuration space
					enabled = enabled && data[order[index]].spaceUri == spaceUri;
				}
				deliverButton.set("enabled", enabled);
			} else {
				deliverButton.set("enabled", false);
				compareButton.set("enabled", false);
			}
			
			if(length > 2){
				ncFactory.createInfo({
					title: this.messages.warningTooManySelectionsTitle,
					subtitle: this.messages.warningTooManySelectionsSubtitle,
					description: this.messages.warningTooManySelectionsDescription,
					solution: this.messages.warningTooManySelectionsSolution,
					closeOnEvent: {context: this, event: "onRowSelectionChanged"}
				}).dispatch();
			}
			
		}));

		return actionsBar.domNode;
	},
	
	buildTreeGridTableNode: function __buildTreeGridTableNode(){
		this._gridTable = new com.ibm.team.vvc.server.core.ui.explorer.VvcExplorerTable();
		return this._gridTable.domNode;
	},
	
	refresh: function __refresh(/*boolean*/hardRefresh){
		this._gridTable.refresh();
		this._idRowMap = {};
		
		this._initializeDatastore();
				
		var query = {
			viewpoint: this._currentView,
			includeArchived: this._showArchived
		}
		var rootViewDef = this._viewStructureCache.get("roots", query);
		dojo.when(rootViewDef, dojo.hitch(this, function __viewStructureCacheGet(rootView){
			var rootUris = [];
			// Convert to an array.  The "roots" macro is a special case
			for(var rootUri in rootView){
				rootUris.push(rootUri);
			}
			this._configurationCache.getAll(rootUris).then(dojo.hitch(this, function __configurationCacheGetAll(confs){
				var orderedConfs = [];
				for(var confUri in rootView){
					if(confs[confUri]){
						orderedConfs.unshift(confs[confUri]);
					} else {
						// There was an error getting this one
					}
				}
				this._gridTable.addRows(orderedConfs);
			}));
		}));

	},

	showArchived: function __showArchived(/*boolean*/showArchived){
		this._showArchived = showArchived;
		this._viewStructureCache.clear();
		this.refresh();
	},
	
	setFlowView: function __setFlowView(){
		return this._setView(vvcConstants.EXPLORER_FLOW_VIEW);
	},
	
	setDependencyView: function __setDependencyView(){
		return this._setView(vvcConstants.EXPLORER_DEPENDENCY_VIEW);
	},
		
	_setView: function __setView(view){
		this._currentView = view;
		this.refresh();
		return null;
	},
	
	/*
	 * This is specific to the VVC Explorer.  If it's needed elsewhere, move it into the 
	 * VvcButtonFactory.
	 */
	_getViewpointsButton: function __getViewpointsButton(){
		var menu = new jazz.ui.Toolbar({
			"class": "actionMenuPopout",
		});
		
		var flowViewButton = new jazz.ui.toolbar.Button({
			imgClass: "icons-compare",
			title: "Flow View",
			label: "Flow View",
			onClick: dojo.hitch(this, function(){
				button.set("label", "Flow View");
				this.setFlowView();
			}),
		});
		
		var dependencyViewButton = new jazz.ui.toolbar.Button({
			imgClass: "icons-compare",
			title: "Dependency View",
			label: "Dependency View",
			onClick: dojo.hitch(this, function(){
				this.setDependencyView();
			}),
		});
		
		menu.addChild(flowViewButton);
		menu.addChild(dependencyViewButton);
		
		var button = new jazz.ui.toolbar.MenuButton({
			menuProvider: function(){
				return menu;
			},
			cacheMenu: true,
			title: "View",
			label: "View",
			imgClass: "icons-show-actions",
		});
		return button;
	},
	
	onRefreshRowById: function __onRefreshRowById(id){
		// Clear out the caches
		this._configurationCache.clear(id);
		this._viewStructureCache.clear(id);
		var rows = this._idRowMap[id];
		delete this._idRowMap[id];
		this._idRowMap[id] = [];
		var rowBusyIndicatorMap = [];
		dojo.forEach(rows, dojo.hitch(this, function __rowsForEach(row, rows, index){
			rowBusyIndicatorMap[row] = row.showBusyIndicator();
		}));
		
		// Get the new conf data
		var conf, view;
		var getConfDef = this._configurationCache.get(id);	
		getConfDef.then(dojo.hitch(this, function __getConfDefThen(result){
			conf = result;
		}));
		
		// Get the new view data.  Views need specialized queries.
		var getViewDef = this._viewStructureCache.get(id, {
			viewpoint: this._currentView, 
			includeArchived: this._showArchived,
			uri: id,
		});
		getViewDef.then(dojo.hitch(this, function __getViewDefThen(result){
			view = result;
		}));
		
		// Confs and views are fetched async & independently, 
		// need to wait for both to resolve before continuing
		var readyDef = new dojo.DeferredList([getConfDef, getViewDef, ]);
		readyDef.then(dojo.hitch(this, function __readyDefThen(){
			dojo.forEach(rows, dojo.hitch(this, function __rowsForEach(row, index, rows){
				var expanded = row._isExpanded;
				row.collapse();
				conf.children = view;
				var newRow = this._gridTable._buildRow(conf, row._childOf);
				dojo.place(newRow.domNode, row.domNode, "replace");
				rowBusyIndicatorMap[row].resolve(true);
				row._removeRow();
				newRow._isOnTheDom.resolve(true);
				if(expanded){
					newRow.expand();
				}
			}));
		}));
	},
	
	onExpandRow: function __onExpandRow(row, data, childOf){
		/*
		 * Need to guard against a race condition:  User expands row, then 
		 * quickly collapses it.  If the row needs to goto the server or takes
		 * some time to render, it may still appear, even though the parent row
		 * has been collapsed!
		 */
		if(row._isExpanded){
			var busyIndicator = row.showBusyIndicator();
			var structureQuery = {
				viewpoint: this._currentView, 
				includeArchived: this._showArchived,
				uri: data.uri,
				configurationSpace: data.spaceUri
			};
			
			this._viewStructureCache.get(data.uri, structureQuery).then(dojo.hitch(this, function ___viewStructureCacheGet(childUris){
				// Confs and views are retrieved asyonc & independently, need containers for the retrieved data.
				var confs, views;
				
				// Get all the confs
				var confsDef = this._configurationCache.getAll(childUris);
				confsDef.then(dojo.hitch(this, function __confsDefThen(results){
					confs = results;
				}));
				
				// Get all the views
				var viewQueries = [];
				for(var i = 0; i < childUris.length; i++){
					// Views require specialised query data
					viewQueries.push({
						viewpoint: this._currentView, 
						includeArchived: this._showArchived,
						uri: childUris[i],
						configurationSpace: data.spaceUri
					});
				}
				var viewsDef = this._viewStructureCache.getAll(childUris, viewQueries);
				viewsDef.then(dojo.hitch(this, function ___viewStructureCacheGetAll(results){
					views = results;
				}));
				
				// Wait for all the confs + views to come back (independent requests)
				var readyDef = new dojo.DeferredList([confsDef, viewsDef, row._isOnTheDom]);
				readyDef.then(dojo.hitch(this, function __readyDefThen(){
					var orderedConfs = [];
					for(var i = 0; i < childUris.length; i++){
						var currConfUri = childUris[i];
						orderedConfs.unshift(confs[currConfUri]);
					}
					// Bulk add (ensures order is preserved)
					this._gridTable.addRows(orderedConfs, row);
					busyIndicator.resolve(true);
				}));
				
				this._expandedIds[data.uri] = true;
				sessionStorage.setItem(vvcConstants.PRESERVE_EXPANSION_STATE_STORAGE_KEY, dojo.toJson(this._expandedIds));
			}));
		}
	}
	
});
})();
