/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.server.core.ui.explorer.VvcExplorerPage");

dojo.require('dojo.i18n');
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Breadcrumbs");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;

dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.require("com.ibm.team.vvc.server.core.ui.explorer.VvcExplorer");

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.explorer", "VvcExplorerMessages");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	
dojo.declare("com.ibm.team.vvc.server.core.ui.explorer.VvcExplorerPage", [dijit._Widget, dijit._Templated], {
	  
	templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/explorer/templates/VvcExplorerPage.html'),
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.explorer", "VvcExplorerMessages"),
	explorer: null,
	_breadcrumbs: null,
	_notificationCenter: null,
	_view: null,
	
	postCreate: function __postCreate(){		
		// Register the page actions
		var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		ActionRegistry.registerAction(vvcConstants.EXPLORE_CONFIGURATIONS, this, "setView");
		this._notificationCenter = new com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController();
		this.notificationCenterNode.appendChild(this._notificationCenter.domNode);
		
		dojo.subscribe("/dojo/hashchanged", this, this.setView);
	},
	
	setView: function __setView(){
		var hash = dojo.hash();
		if(hash){
			var hashParams = dojo.queryToObject(hash);
			var view = hashParams.view;
			if(view){
				if(view === vvcConstants.EXPLORER_DEPENDENCY_VIEW){
					return this._setView(vvcConstants.EXPLORER_DEPENDENCY_VIEW);
				}
			}
		}
		// Default or view === vvcConstants.EXPLORER_FLOW_VIEW
		return this._setView(vvcConstants.EXPLORER_FLOW_VIEW);
	},

	_setView: function __setView(view){
		if(this._view == view){
			return;
		}
		this._view = view;
		if(!this.explorer){
			this.explorer = new com.ibm.team.vvc.server.core.ui.explorer.VvcExplorer({
				notificationCenter: this._notificationCenter
			});
			this.explorerNode.appendChild(this.explorer.domNode);
		}
		this._constructBreadcrumbs(this._view);
		return this.explorer._setView(this._view);
	},
	
	_constructBreadcrumbs: function __constructBreadcrumbs(view) {
		var parts = [];
		
		var explorerPageUri = PageUtil.createLink(vvcConstants.EXPLORE_CONFIGURATIONS, null, view);
		segment = { 
				label: this.messages[view],
				href : explorerPageUri 
		};
		parts.push(segment);
		
		// add breadcrumb from query parameter
		var breadcrumb = PageUtil.getRequestParmValue("breadcrumb");
		if (breadcrumb) {
			breadcrumb = decodeURIComponent(unescape(breadcrumb));
			segment = {
				label: breadcrumb, 
				href : 'javascript:history.back()'
			};
			parts.push(segment);
		}
		
		if(this._breadcrumbs){
			dojo.empty(this.breadcrumbNode);
		}
		
		this._breadcrumbs = new jazz.ui.Breadcrumbs({segments: parts });
		this._breadcrumbs.placeAt(this.breadcrumbNode);
	},
	   					   
    uninitialize: function __uninitialize() {
    	if (this._breadcrumbs) {
			this._breadcrumbs.destroyRecursive();
			delete this._breadcrumbs;
		}
	}		
})   
})();
