/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.vvc.server.core.ui.internal.changesets.ChangesetCompactRendering");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.util.iframe");
dojo.require("jazz.client.xhr");
dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.changesets", "ChangesetCompactRenderingMessages");


dojo.declare("com.ibm.team.vvc.server.core.ui.internal.changesets.ChangesetCompactRendering", [dijit._Widget, dijit._Templated], {
	widgetsInTemplate: true,

	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.changesets", "ChangesetCompactRenderingMessages"),
	
	templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/internal/changesets/templates/ChangesetCompactRendering.html'),
	
	postCreate: function() {
		this.inherited(arguments);
		
		if (location.search.length > 0) {
			var obj = dojo.queryToObject(location.search.substring(1));
			if (obj) {
				if (obj.changesetUri) {
					var context1 = {
						context: obj.changesetUri,
						messages: this.messages,
						self: this
					};
				
					var args = {
							headers: {
								"Accept": "application/json"
							},
							url: obj.changesetUri + '/details?right_now='+new Date().getTime(),
							handleAs: "json",
							hasBody: false,
						};

					var deferred1 = jazz.client.xhrGet(args);
					deferred1.then(dojo.hitch(context1, function(response, ioArgs) {
						
						var main = response[this.context];
						
						//TODO: space
						
						if (!main) {
							dojo.addClass(this.self.tableNode, "changeset-compact-rendering-hidden");
							dojo.addClass(this.self.errorNode, "changeset-compact-rendering-visible");
							return;
						}
						
						if (main[Constants.CHANGESET_CREATOR] && main[Constants.CHANGESET_CREATOR][0]) {
							try {
								var creator = main[Constants.CHANGESET_CREATOR][0].value;
								
								var index = creator.lastIndexOf('/');
								
								this.self.creatorNode.appendChild(dojo.create("a", {href: creator, innerHTML: creator.substring(index + 1)}));
							} catch (err) {
								this.self.creatorNode.appendChild(dojo.doc.createTextNode(this.messages.unknown));
							}
						}

						if (main[Constants.CHANGESET_CREATED] && main[Constants.CHANGESET_CREATED][0]) {
							try {
								var created = main[Constants.CHANGESET_CREATED][0].value;
								created = new Date(created).toDateString();
								this.self.createdNode.appendChild(dojo.doc.createTextNode(created));
							} catch (err) {
								this.self.createdNode.appendChild(dojo.doc.createTextNode(this.messages.unknown));
							}
						}
						
						if (main[Constants.STATE] && main[Constants.STATE][0]) {
							var state = main[Constants.STATE][0].value;
							
							if (state === Constants.STATE_COMMITTED) {
								state = this.messages.completenessCommitted;
							} else if (state === Constants.STATE_COMPLETED) {
								state = this.messages.completenessCompleted;
							} else if (state === Constants.STATE_IN_PROGRESS) {
								state = this.messages.completenessInProgress;
							} else {
								state = this.messages.unknown;
							}
							
							this.self.stateNode.appendChild(dojo.doc.createTextNode(state));
						}
						
					}),
					dojo.hitch(context1, function(responseerror) {
						dojo.addClass(this.self.tableNode, "changeset-compact-rendering-hidden");
						dojo.addClass(this.self.errorNode, "changeset-compact-rendering-visible");
					}));
					
				} else {
					dojo.addClass(this.tableNode, "changeset-compact-rendering-hidden");
					dojo.addClass(this.errorNode, "changeset-compact-rendering-visible");
				}
			}
		
		}
		
		if (window.postMessage && window.parent) {
			var newPreviewHeight = jazz.util.iframe.getDocumentHeight();
			var resizeMessage = "oslc-preview-height:" + newPreviewHeight;
			window.parent.postMessage(resizeMessage, "*");
			resizeMessage = "jazz-compact-rendering-preview-height:" + newPreviewHeight;
			window.parent.postMessage(resizeMessage, "*");
		} 
	}
	
});

})();
