/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.server.core.ui.internal.changesets.ChangesetsExplorerTable");

dojo.require('dojo.i18n');
dojo.require("dijit._Widget");

dojo.require("jazz.ui.toolbar.Separator");

dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory");
var vvcButtonFactory = com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.ChangesetUiNode");
var ChangesetUiNode = com.ibm.team.vvc.shared.web.ui.util.ChangesetUiNode;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.UserUiNode");
var UserUiNode = com.ibm.team.vvc.shared.web.ui.util.UserUiNode;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.ChangesetResourceUiNode");
var ChangesetResourceUiNode = com.ibm.team.vvc.shared.web.ui.util.ChangesetResourceUiNode;

dojo.require("jazz.ui.ResourceLink");
var ResourceLink = jazz.ui.ResourceLink;

dojo.require("com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils");
var CLMUtils = com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTable");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.changesets", "ChangesetsPageMessages");
	
dojo.declare("com.ibm.team.vvc.server.core.ui.internal.changesets.ChangesetsExplorerTable", [com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTable], {
	
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.changesets", "ChangesetsPageMessages"),
	
	/**
	 * Registers all the row builders with this table.  The order in which they are registered will also reflect
	 * the order in which the columns appear.
	 */
	registerRowBuilders: function(){
		builders = [];
		// Order matters!
		builders.push(this._actionsBuilder());
		builders.push(this._nameBuilder());
		builders.push(this._creatorBuilder());
		builders.push(this._workItemLinksBuilder());
		return builders;
	},
		
	/**
	 * Builds the actions column for a row
	 */
	_actionsBuilder: function(tr, data, table){
		var builder = {
			columnName: this.messages.actionsColumnTitle,
			columnClass: "actionsColumn",
			builderFunc: dojo.hitch(this, function(tr, data, table){
				var readyDeferred = new dojo.Deferred();
				var superReadyDeferred = this._defaultBuilder(tr, data, table);
				dojo.when(superReadyDeferred, dojo.hitch(this, function(td){										
					if(tr._childOf){
						this._resourceActionsBuilder(tr, data, table, td);
					} else {
						this._changesetActionsBuilder(tr, data, table, td)
					}
					readyDeferred.resolve(td);
				}));
				return readyDeferred;
			})
		};
		return builder;
	},
	
	/**
	 * Builds the name column for the row
	 */
	_nameBuilder: function(tr, data, table){
		var builder = {
			columnName: this.messages.nameColumnTitle,
			columnClass: "changesetNameColumn",
			builderFunc: dojo.hitch(this, function(tr, data, table){
				var readyDeferred = new dojo.Deferred();
				var superReadyDeferred = this._defaultBuilder(tr, data, table);
				dojo.when(superReadyDeferred, dojo.hitch(this, function(td){	
					if(tr._childOf){
						this._resourceNameBuilder(tr, data, table, td);
					} else {
						this._changesetNameBuilder(tr, data, table, td);
					}
					readyDeferred.resolve(td);
				}));
				return readyDeferred;
			})
		};
		return builder;
	},
	
	_creatorBuilder: function(tr, data, table){
		var builder = {
			columnName: this.messages.creatorColumnTitle,
			columnClass: "creatorColumn",
			builderFunc: dojo.hitch(this, function(tr, data, table){
				var readyDeferred = new dojo.Deferred();
				var superReadyDeferred = this._defaultBuilder(tr, data, table);
				dojo.when(superReadyDeferred, dojo.hitch(this, function(td){										
					if(!tr._childOf){
						var userUiNode = new UserUiNode({
							uri: data.creatorUri
						});
						td.appendChild(userUiNode.domNode);
					}
					readyDeferred.resolve(td);
				}));
				return readyDeferred;
			})
		};
		return builder;
	},
	
	/**
	 * Builds the work item links area
	 */
	_workItemLinksBuilder: function(tr, data, table){
		var builder = {
			columnName: this.messages.workItemLinksColumnTitle,
			columnClass: "workItemLinks",
			builderFunc: dojo.hitch(this, function(tr, data, table){
				var readyDeferred = new dojo.Deferred();
				var superReadyDeferred = this._defaultBuilder(tr, data, table);
				dojo.when(superReadyDeferred, dojo.hitch(this, function(td){
					if(tr._childOf){
						this._resourceLinksBuilder(tr, data, table, td);
					} else {
						this._changesetLinksBuilder(tr, data, table, td);
					}
					readyDeferred.resolve(td);
				}));
				return readyDeferred;
			})
		};
		return builder;
	},
	
	_changesetActionsBuilder: function(tr, data, table, td){
		var gc = tr._gc;
		
		var toolbar = new jazz.ui.Toolbar({
			"class": "actionsToolbar",
			onfocus: dojo.hitch(this, function(){tr.onFocus()}),
			onblur: dojo.hitch(this, function(){tr.onBlur()})
		});
		gc.destroys(toolbar);
									
		var showNextSeparator = false;
		
		/*
		 * TODO: Support (server & client side)for per-row refresh for
		 * change sets. For now, all calls to refresh the
		 * individual row will refresh the entire table instead.
		 */ 
		
		// Discard
		if(data.state !== Constants.STATE_COMMITTED){
			var discardChangesetButton = vvcButtonFactory.getDiscardChangesetButton(data, {
				actionCallback: dojo.hitch(this, function(){
					tr.onRefresh();
				})						
			});
			toolbar.addChild(discardChangesetButton);
			
			// Separator
			toolbar.addChild(new jazz.ui.toolbar.Separator());
		}		
		
		// Share
		if(data.state !== Constants.STATE_COMMITTED){
			var shareChangesetButton = vvcButtonFactory.getShareChangesetButton(data, {
				actionCallback: dojo.hitch(this, function(){
					tr.onRefresh();
				})						
			});
			toolbar.addChild(shareChangesetButton);
			showNextSeparator = true;
		}
		
		// Complete
		if(data.state === Constants.STATE_IN_PROGRESS){
			var completeChangesetButton = vvcButtonFactory.getCompleteChangesetButton(data, {
				actionCallback: dojo.hitch(this, function(){
					tr.onRefresh();
				})						
			});
			toolbar.addChild(completeChangesetButton);
			showNextSeparator = true;
		}
		
		// Reverse
		if(data.state === Constants.STATE_COMMITTED){
			var reverseChangesetButton = vvcButtonFactory.getReverseChangesetButton(data, {
				actionCallback: dojo.hitch(this, function(){
					tr.onRefresh();
				})						
			});
			toolbar.addChild(reverseChangesetButton);
			showNextSeparator = true;
		}
		
		// Separator
		if(showNextSeparator){
			toolbar.addChild(new jazz.ui.toolbar.Separator());
		}
		
		// Rename
		var renameChangesetButton = vvcButtonFactory.getRenameChangesetButton(data, {
			actionCallback: dojo.hitch(this, function(){
				tr.onRefresh();
			})						
		});
		toolbar.addChild(renameChangesetButton);
		
		// Link to work item
		var linkWorkitemToChangesetButton = vvcButtonFactory.getLinkChangeRequestToChangesetButton(data, {
			actionCallback: dojo.hitch(this, function(){
				tr.onRefresh();
			})						
		});
		toolbar.addChild(linkWorkitemToChangesetButton);
		
		td.appendChild(toolbar.domNode);
	},
	
	_resourceActionsBuilder: function(tr, data, table, td){
		if(data.showNoResults){
			return;
		}
		var gc = tr._gc;
		
		var toolbar = new jazz.ui.Toolbar({
			"class": "actionsToolbar",
			onfocus: dojo.hitch(this, function(){tr.onFocus()}),
			onblur: dojo.hitch(this, function(){tr.onBlur()})
		});
		gc.destroys(toolbar);
		
		var showHistoryButton = vvcButtonFactory.getShowHistoryButton({
			conceptUri: data.concept
		});
		toolbar.addChild(showHistoryButton);
		gc.destroys(showHistoryButton);
		
		td.appendChild(toolbar.domNode);
	},
	
	_changesetNameBuilder: function(tr, data, table, td){
		var gc = tr._gc;

		var expando = new com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTableExpando({
			explorerTable: this,
			tableRow: tr
		});
		td.appendChild(expando.domNode);
		
		var configuratonTitleNode = new ChangesetUiNode({
			changeset: data
		}, td);
		gc.destroys(configuratonTitleNode);
		
		gc.disconnects(dojo.connect(configuratonTitleNode.domNode, 'onfocus', dojo.hitch(this, function(){
			tr.onFocus();
		})));
		gc.disconnects(dojo.connect(configuratonTitleNode.domNode, 'onblur', dojo.hitch(this, function(){
			tr.onBlur();
		})));
		
		gc.disconnects(dojo.connect(configuratonTitleNode, 'onfocus', dojo.hitch(this, function(){
			tr.onFocus();
		})));
		gc.disconnects(dojo.connect(configuratonTitleNode, 'onblur', dojo.hitch(this, function(){
			tr.onBlur();
		})));		
	},
	
	_resourceNameBuilder: function(tr, data, table, td){
		var childIndenter = dojo.create("span", {
			// Directly setting the style here is a secessary evil since we need to generate the width.
			style: "width:40px; height:26px; display:inline-block;",
			innerHTML: "&nbsp;"
		});
		td.appendChild(childIndenter);
		
		if(data.showNoResults){
			var noResultsNode = dojo.create("span", {
				innerHTML: this.messages.noResourcesFound,
				"class": "noResultsNode"
			}, td);
		} else {		
			var resourceNode = new ChangesetResourceUiNode({
				"data": data
			});
			td.appendChild(resourceNode.domNode);
		}
	},
	
	_changesetLinksBuilder: function(tr, data, table, td){
		var workItemWidget = new com.ibm.team.vvc.server.core.ui.internal.changesets.ChangesetsExplorerTable._workItemWidgetController(data);
		td.appendChild(workItemWidget.domNode);
	},
	
	_resourceLinksBuilder: function(tr, data, table, td){
		// Stub, does nothing...
	}
	

});

dojo.declare("com.ibm.team.vvc.server.core.ui.internal.changesets.ChangesetsExplorerTable._workItemWidgetController", dijit._Widget, {
	
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.changesets", "ChangesetsPageMessages"),
	_changeset: null,
	
	constructor: function(changeset){
		this._changeset = changeset;
	},
	
	postCreate: function(){
		this._showAuthenticationNode();
		var authenticationDeferred = new dojo.Deferred();
		CLMUtils.authenticate(authenticationDeferred);
		authenticationDeferred.then(dojo.hitch(this, function(result){
			this._showLoadingNode();
			this._getWorkItemLinks().then(dojo.hitch(this, function(results){
				if(results && results.length > 0){
					dojo.empty(this.domNode);
					var sorted = this._sortWorkItemLinks(results);
					for(var i = 0; i < sorted.length; i++){
						var widget = new com.ibm.team.vvc.server.core.ui.internal.changesets.ChangesetsExplorerTable._workItemWidget(sorted[i]);
						this.domNode.appendChild(widget.domNode);
						if(i < (sorted.length - 1)){
							dojo.create("span", {
								innerHTML: ", "
							}, this.domNode);
						}
					}
				} else {
					this._showNoResultsFoundNode();
				}
			}), dojo.hitch(this, function(error){
				this._showErrorNode();
			}));
		}), dojo.hitch(this, function(error){
			this._showErrorNode();
		}));
	},
	
	_showAuthenticationNode: function(){
		dojo.empty(this.domNode);
		dojo.create("span", {
			innerHTML: this.messages.waitingForAuthentication,
			"class": "noResultsNode"
		}, this.domNode);
	},
	
	_showLoadingNode: function(){
		dojo.empty(this.domNode);
		dojo.create("span", {
			innerHTML: this.messages.loading,
			"class": "noResultsNode"
		}, this.domNode);
	},
	
	_showNoResultsFoundNode: function(){
		dojo.empty(this.domNode);
		dojo.create("span", {
			innerHTML: this.messages.noWorkitemLinks,
			"class": "noResultsNode"
		}, this.domNode);
	},
	
	_showErrorNode: function(){
		dojo.empty(this.domNode);
		dojo.create("span", {
			innerHTML: this.messages.error,
			"class": "error"
		}, this.domNode);
	},
	
	_getWorkItemLinks: function(){
		var urlArgs = {
			changeset: this._changeset.uri,
			consumerUrl: VvcUtils.getVvcUri()
		}
		
		var getLinksDeferred = jazz.client.xhrGet({
			headers: {
				"Content-Type": "application/x-oslc-common-link+xml",
				"Accept": "application/atom+json"
			},
			url: VvcUtils.getVvcUri() + "/changesetLinks?" + dojo.objectToQuery(urlArgs),
			handleAs: "json",
			hasBody: false
		});
		return getLinksDeferred;
	},
	
	_sortWorkItemLinks: function(links){
		var sorted = [];
		var toBeAdded;
		while(links.length > 0){
			toBeAdded = links.pop();
			var targetAdding = toBeAdded.target;
			var numberAdding = targetAdding.substring(dojo.lastIndexOf(targetAdding, '/') + 1);
			for(var i = 0; i < sorted; i++){
				var targetAdded = sorted[j];
				var numberAdded = targetAdded.substring(dojo.lastIndexOf(targetAdded, '/') + 1);
				if(numberAdding < numberAdded){
					sorted.splice(j, 1, toBeAdded);
					toBeAdded = null;
					break;
				}
			}
			
			if(toBeAdded){
				sorted.push(toBeAdded);
			}
		}
		return sorted;
	},
	
});

dojo.declare("com.ibm.team.vvc.server.core.ui.internal.changesets.ChangesetsExplorerTable._workItemWidget", dijit._Widget, {

	_workItem: null,
	_workItemDetails: null,
	_showAbbreviation: null,
	
	constructor: function(workItem){
		this._workItem = workItem;
		this._showAbbreviation = true;
	},
	
	postCreate: function(){
		this.domNode = dojo.create("span");
		this._showCompactNode();
	},
	
	_showCompactNode: function(){
		this._showAbbreviation = true;
		this._refresh();
	},
	
	_showFullNode: function(){
		this._showAbbreviation = false;
		this._refresh();
	},
	
	_refresh: function(){
		dojo.empty(this.domNode);
		var target = this._workItem.target;
		var linkIconNode = dojo.create("span", {
			innerHTML: "&nbsp;",
		}, this.domNode);
		var title = target.substring(dojo.lastIndexOf(target, '/') + 1);
		var link = new ResourceLink({
			uri: this._workItem.target,
			label: title,
			iconNode: linkIconNode,
			useAbbreviation: this._showAbbreviation
		});
		this.domNode.appendChild(link.domNode);
	},
	
	_getWorkitemDetails: function(){
		if(this._workItemDetails){
			return this._getWorkitemDetails();
		}
		var workItemDef = jazz.client.xhrGet({
			url: links[i].target + "?oslc.properties=dcterms:title",
			headers: {
				"Accept": "application/rdf+xml",
				"OSLC-Core-Version": "2.0"
			},
			handleAs: "xml", //$NON-NLS-1$
		});
		workItemDef.then(dojo.hitch(this, function(details){
			this._workItemDetails = details;
		}), dojo.hitch(this, function(error){
			ncFactory.createXhrError({xhrError: response, ioArgs: ioArgs}).dispatch();
		}));
		return workItemDef;
	}

});

})();
