/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
dojo.provide("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTable");

dojo.require('dojo.i18n');
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Toolbar")
dojo.require("jazz.ui.toolbar.MenuButton");
dojo.require("jazz.ui.toolbar.Separator");

dojo.require("com.ibm.team.vvc.shared.web.ui.gc.VvcGarbageCollector");
var GC = com.ibm.team.vvc.shared.web.ui.gc.VvcGarbageCollector;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;

dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory");
var vvcButtonFactory = com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory;

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.grid", "VvcTreeGridMessages");
	
dojo.declare("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTable", [dijit._Widget], {
	
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.grid", "VvcTreeGridMessages"),
	_columnBuilders: null,
	_rowBuilderFuncs: null,
	_table: null,
	_tableHead: null,
	_tableBody: null,
	_gc: null,
	/*
	 * To prevent browser crashes and improve perceived performance 
	 * during bulk operations, rows will be added to the browser in 
	 * chunks.
	 */
	_bulkAddRowsChunkSize: 25,
	
	constructor: function __VvcTreeGridTable(){
		this._columnBuilders = [];
		this._gc = new GC();
	},

	postCreate: function __postCreate(){
		// Register the builders once
		this._columnBuilders = this.registerRowBuilders();
		
		// Build initial table
		this.refresh();
	},
	
	destroy: function __destroy(){
		this.inherited(arguments);
		this.columnBuilders = null;
		this.rowBuilderFuncs = null;
		this.table = null;
		this.tableHead = null;
		this.tableBody = null;
		this.gc = null;
		this.domNode = null;
		this._gc.destroy();
	},
	
	/**
	 * Refreshes the tree grid table.  This will build the table off-screen, then replace the visible table.
	 */
	refresh: function __refresh(){
		var table = dojo.create("table");
		this._gc.destroys(table);
		dojo.addClass(table, "vvcTreeGrid");
		this._buildTableHeaders(table);
		this._buildTableContent(table);
		
		// Now add the table to the DOM, destructively repalcing the existing one
		this._table = table;
		dojo.empty(this.domNode);
		this.domNode.appendChild(this._table);
	},
	
	/**
	 * Builds the headers for the table, based on the registered builders.
	 */
	_buildTableHeaders: function __buildTableHeaders(table){
		this._tableHead = dojo.create("thead");
		this._gc.destroys(this._tableHead);
		var tr = dojo.create("tr");
		this._gc.destroys(tr);
		dojo.addClass(this._tableHead, "vvcTreeGridColumnHeadersRow");
		// Must preserve the order, so forEach loops are not a good choice
		var columnNames = this._getBuilderAttr("columnName");
		var columnClasses = this._getBuilderAttr("columnClass");
		var length = columnNames.length;
		for(var index = 0; index < length; index++){
			var th = dojo.create("th");
			this._gc.destroys(th);
			dojo.addClass(th, "vvcTreeGridColumnHeader");
			dojo.addClass(th, columnClasses[index]);
			th.appendChild(dojo.doc.createTextNode(columnNames[index]));
			tr.appendChild(th);
		}
		
		// Add the headers to the table
		this._tableHead.appendChild(tr);
		table.appendChild(this._tableHead);
	},
	
	/**
	 * If the datasource exists, build the table.  Otherwise, do nothing.
	 */
	_buildTableContent: function __buildTableContent(table){
		this._tableBody = dojo.create("tbody");
		this._gc.destroys(this._tableBody);
		table.appendChild(this._tableBody);
	},
	
	/**
	 * Adds a row
	 * 
	 * @param data (required) Data to build the row
	 * @param childOfRow (optional) New row is a child of this row
	 * 
	 * @return A new vvcTreeGridRow
	 */
	addRow: function __addRow(data, childOf){
		var row = this._buildRow(data, childOf);
		/*
		 * Need to guard against a race condition:  User expands childOf, then 
		 * quickly collapses it.  If the row needs to goto the server or takes
		 * some time to render, it may still appear, even though the parent row
		 * has been collapsed!
		 */
		if(childOf && !childOf._isExpanded){
			return null;
		}
		this._insertRow(row, childOf);		
		return row;
	},
	
	/**
	 * Adds multiple rows
	 * 
	 * @param data (required) Data[] to build the rows
	 * @param childOfRow (optional) New rows are children of this row
	 * 
	 * @return A new vvcTreeGridRow[]
	 */
	addRows: function __addRows(data, childOf){
		var rows = [];
		var length = data.length;
		var chunks = parseInt(length / this._bulkAddRowsChunkSize) + 1;
		var insertAfter = childOf ? childOf : null;
		for(var chunkIndex = 0; chunkIndex < chunks; chunkIndex++){
			var chunkStart = chunkIndex * this._bulkAddRowsChunkSize;
			var chunkFinish = (chunkIndex + 1) * this._bulkAddRowsChunkSize;
			for(var index = chunkStart; index < chunkFinish && index < length; index++){
				rows[index] = this._buildRow(data[index], childOf);
				
				/*
				 * Need to guard against a race condition:  User expands childOf, then 
				 * quickly collapses it.  If the row needs to goto the server or takes
				 * some time to render, it may still appear, even though the parent row
				 * has been collapsed!
				 */
				if(childOf && !childOf._isExpanded){
					// Re-run the collapse code to make sure all other children are destroyed
					childOf.collapse();
					return [];
				}
			}
			// Insert the chunk into the DOM
			for(var index = chunkStart; index < chunkFinish && index < length; index++){
				row = rows[index];
				this._insertRow(row, insertAfter);
				insertAfter = row;
			}
		}
		return rows;
	},
	
	/**
	 * Builds a row
	 * 
	 * @param data (required) Data to build the row
	 * @param childOf (optional) New row is a child of this row
	 * 
	 * @return A new vvcTreeGridRow.
	 */
	_buildRow: function __buildRow(data, childOf){
		var row = new com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTableRow({
			vvcTreeGrid: this,
			data: data,
			columnBuilders: this._columnBuilders,
			"childOf": childOf
		});	
		
		this._gc.destroys(row);

		this.onCreateRow(row, data, childOf);
		
		return row;
	},
	
	/**
	 * Inserts a row
	 */
	_insertRow: function __insertRow(row, afterRow){
		// Insert into the DOM
		if(afterRow){
			dojo.place(row.domNode, afterRow.domNode, "after");
		} else {
			this._tableBody.appendChild(row.domNode);
		}
		row._isOnTheDom.resolve(true);
	},
		
	/**
	 * Registers all the row builders with this table.  The order in which they are registered will also reflect
	 * the order in which the columns appear.
	 */
	registerRowBuilders: function __registerRowBuilders(){
		// Flyweight stub
		return [];
	},
	
	/**
	 * Helper method, returns an array of some part of the column builders
	 */
	_getBuilderAttr: function __getBuilderAttr(attr){
		var results = [];
		var length = this._columnBuilders.length;
		for(var index = 0; index < length; index++){
			var builder = this._columnBuilders[index];
			results[index] = builder[attr];
		}
		return results;
	},
	
		
	/**
	 * Builds a blank table cell and returns it.  Subclasses can override accordingly.
	 */
	_defaultBuilder: function __defaultBuilder(tr, data, table){
		var td = dojo.create("td");
		this._gc.destroys(td);
		dojo.addClass(td, "vvcTreeGridCell");
		// Apply default stuff here
		return td;
	},
	
	/**
	 * Event handler for when a row is created.  Allows observers to hook up
	 * events to the row. 
	 */
	onCreateRow: function __onCreateRow(row, data, childOf){
		/* Stub */
	},
	
	/**
	 * Event handler for when the whole table needs to be refreshed.
	 */
	onRefresh: function __onRefresh(){
		/* Stub */
	},
		
});
	
dojo.declare("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTableRow", [dijit._Widget], {
	  
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.grid", "VvcTreeGridMessages"),
	_vvcTreeGrid: null,
	_data: null,
	_columnBuilders: null,
	_ready: null,
	_childOf: null,
	_childLevel: null,	// 0 indicates not a child, 1 indicates
	_isExpanded: null,
	_gc: null,
	_isOnTheDom: null,
	
	/**
	 * A generic row for the VVC tree grid.
	 *
	 * @param args.data (required) The data to build the row against.  Must make sense to the <code>args.builders</code> row builders.
	 * @param args.vvcTreeGrid (required) The controlling tree grid table
	 * @param args.builders (required) Array of buildres to invoke.  Order will be preserved.
	 */
	constructor: function __VvcTreeGridTableRow(args){
		this._ready = new dojo.Deferred();

		// Required args!
		this._data = args.data;
		this._vvcTreeGrid = args.vvcTreeGrid;
		this._columnBuilders = args.columnBuilders;
		this._childOf = args.childOf;
		this._childLevel = this._childOf ? this._childOf._childLevel + 1 : 0;
		this._isExpanded = false;
		this._gc = new GC();
		this._isOnTheDom = new dojo.Deferred();
	},
	
	postCreate: function __postCreate(){
		this._buildRow();
	},
	
	destroy: function __destroy(){
		this.inherited(arguments);
		this._childOf = null;
		this._columnBuilders = null;
		this._data = null;
		this._vvcTreeGrid = null;
		this.domNode = null;
		this._gc.destroy();
		this._gc = null;
		this._isOnTheDom = null;
	},
	
	/**
	 * Invokes all the registered row builders in order, and builds up the row.
	 * 
	 * @return The DOM for a row, ready to be inserted.
	 */
	_buildRow: function __buildRow(){
		var tr = dojo.create("tr");
		this._gc.destroys(tr);
		
		dojo.addClass(tr, "vvcTreeGridRow");
		var columnBuilderLength = this._columnBuilders.length;
		
		var defList = [];
		for(var index = 0; index < columnBuilderLength; index++){
			var columnBuilder = this._columnBuilders[index]["builderFunc"];
			var def = columnBuilder(this, this._data, this._vvcTreeGrid);
			defList[index] = def;
		}
		
		this._readyList = new dojo.DeferredList(defList);
		
		dojo.when(this._readyList, dojo.hitch(this, function __rowIsReady(results){
			var length = results.length;
			for(var index = 0; index < length; index++){
				var td = results[index][1];
				tr.appendChild(td);
				dojo.addClass(td, this._columnBuilders[index]["columnClass"]);
			}
			this.domNode = tr;
			this._ready.resolve(true);
		}));
		this._gc.destroys(this._ready);
		
		return this._ready;
	},
	
	_removeRow: function __removeRow(){
		this.onRemove();
		dojo.destroy(this.domNode);
		this.destroy();
	},
		
	/**
	 * Indicates when this row is ready.
	 * 
	 * @return <code>true</code> if the row is ready, <code>false</code> if it is not or has failed,
	 * 			and a <code>dojo.Deferred</code> if it's in progress.
	 */
	isReady: function __isReady(){
		return this._ready;
	},
	
	/**
	 * Programatically expand the row.
	 */
	expand: function __expand(){
		/* Stub */
	},
	
	/**
	 * Event handler for when the row is expanded.
	 */
	onExpand: function __onExpand(/*boolean*/skipCache){
		/* Stub */
	},
	
	/**
	 * Programatically collapse the row.
	 */
	collapse: function __collapse(){
		/* Stub */
	},
	
	/**
	 * Event handler for when the row is collapsed.
	 */
	onCollapse: function __onCollapse(){
		/* Stub */
	},
	
	/**
	 * Programatically select the row.
	 */
	select: function __select(){
		/* Stub */
	},
	
	/**
	 * Event handler for when the row is selected.
	 */
	onSelect: function __onSelect(){
		/* Stub */
	},
	
	/**
	 * Programatically select the row.
	 */
	deselect: function __deselect(){
		/* Stub */
	},
	
	/**
	 * Event handler for when the row is deselected.
	 */
	onDeselect: function __onDeselect(){
		/* Stub */
	},
	
	/**
	 * Event handler for when the row is removed.  Invoked before
	 * the row is actually removed.
	 */
	onRemove: function __onRemove(){
		/* Stub */
	},
	
	/**
	 * Signals the table that this row needs to be refreshed.
	 */
	onRefresh: function __onRefresh(){
		/* Stub */
	},
	
	/**
	 * Event handler for when the row has gained focus.
	 */
	onFocus: function __onFocus(){
		/* Stub */
	},
	
	/**
	 * Event handler for when the row has lost focus.
	 */
	onBlur: function __onBlur(){
		/* Stub */
	},
	
	/**
	 * Shows an indicator while the row is busy doing something, such as
	 * refreshing, expanding, etc...
	 * 
	 * @return A <code>dojo.Deferred</code> which, when resolved, will
	 * 			hide the busy indicator. 
	 */
	showBusyIndicator: function __showBusyIndicator(){
		var hideLoadingNodeWhenDone = new dojo.Deferred();
		this._onShowBusyIndicator();
		hideLoadingNodeWhenDone.then(dojo.hitch(this, function __hideBusyIndicator(){
			this._onHideBusyIndicator();
		}));
		return hideLoadingNodeWhenDone;
	},
	
	_onShowBusyIndicator: function __onShowBusyIndicator(){
		/* Stub */
	},
	
	_onHideBusyIndicator: function __onHideBusyIndicator(){
		/* Stub */
	}
	
});

dojo.declare("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTableExpando", [dijit._Widget, dijit._Templated], {
	  
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.grid", "VvcTreeGridMessages"),
	templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/internal/grid/templates/VvcTreeGridTableExpando.html'),
	_vvcTreeGrid: null,
	_tableRow: null,
	_gc: null,

	/**
	 * A simple expando (aka "twisty") for the VVC tree grid. 
	 */	
	constructor: function __VvcTreeGridTableExpando(args){
		this._vvcTreeGrid = args.vvcTreeGrid;
		this._tableRow = args.tableRow;
		
		this._gc = new GC();
		
		this._gc.disconnects(dojo.connect(this._tableRow, "expand", dojo.hitch(this, this.expand)));
		this._gc.disconnects(dojo.connect(this._tableRow, "collapse", dojo.hitch(this, this.collapse)));		
	},
	
	destroy: function __destroy(){
		this.inherited(arguments);
		this._tableRow = null;
		this._vvcTreeGrid = null;
		this._gc.destroy();
	},

	onExpand: function __onExpand(event){
		dojo.stopEvent(event);
		this.expand();
	},
	
	expand: function __expand(){
		this._toggle(true);
		this._tableRow.onExpand();
	},
	
	onCollapse: function __onCollapse(event){
		dojo.stopEvent(event);
		this.collapse();
	},
	
	collapse: function __collapse(){
		this._toggle(false);
		this._tableRow.onCollapse();
	},
	
	_toggle: function __toggle(isExpanded){
		if(isExpanded){
			this._toggleCss(this.expandNode, "hidden", "expandNode");
			this._toggleCss(this.collapseNode, "collapseNode", "hidden");
			this.collapseNode.focus();
		} else {
			this._toggleCss(this.expandNode, "expandNode", "hidden");
			this._toggleCss(this.collapseNode, "hidden", "collapseNode");
			this.expandNode.focus();
		}
		this._tableRow._isExpanded = isExpanded;
	},

	_toggleCss: function __toggleCss(node, classToAdd, classToRemove){
		dojo.addClass(node, classToAdd);
		dojo.removeClass(node, classToRemove);
	},
	
	onFocus: function __onFocus(){
		this._tableRow.onFocus();
	},
	
	onBlur: function __onBlur(){
		this._tableRow.onBlur();
	}
});

dojo.declare("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTableBusyIndicator", [dijit._Widget, dijit._Templated], {
	
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.grid", "VvcTreeGridMessages"),
	templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/internal/grid/templates/VvcTreeGridTableBusyIndicator.html'),
	_vvcTreeGrid: null,
	_tableRow: null,
	_gc: null,
		
	/**
	 * A simple busy indicator for the VVC tree grid. 
	 */	
	constructor: function(args){
		this._vvcTreeGrid = args.vvcTreeGrid;
		this._tableRow = args.tableRow;
		
		this._gc = new GC();
		
		this._gc.disconnects(dojo.connect(this._tableRow, "_onShowBusyIndicator", dojo.hitch(this, this.showBusyIndicator)));
		this._gc.disconnects(dojo.connect(this._tableRow, "_onHideBusyIndicator", dojo.hitch(this, this.hideBusyIndicator)));
	},
	
	destroy: function(){
		this.inherited(arguments);
		this._tableRow = null;
		this._vvcTreeGrid = null;
		this._gc.destroy();
	},

	
	showBusyIndicator: function __showBusyIndicator(){
		dojo.addClass(this.domNode, "busyIndicatorIconNode");
	},
	
	hideBusyIndicator: function __hideBusyIndicator(){
		dojo.removeClass(this.domNode, "busyIndicatorIconNode");
	}

});

})();
