/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.page.DependencyPicker");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("jazz.ui.MessageBox");
	dojo.require("jazz.ui.Breadcrumbs")

	dojo.require("net.jazz.ajax.ui.PlatformUI");
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;

	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils");
	var InfoContentUtils = com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils;
	
	dojo.require("jazz.app.ua.PageHoverHelp");
	
	dojo.require("com.ibm.team.vvc.server.core.ui.snapshots.WorkspacesAndSnapshotsViewer");
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "DependencyPickerMessages");
	
	var VVC_APP = "com.ibm.team.vvc.server.core";
	var DEPENDENCY_SERVICE = "dependencies";
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.page.DependencyPicker", [dijit._Widget, dijit._Templated], {

		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl(VVC_APP, "ui/internal/page/templates/DependencyPicker.html"),

		_pickConfigViewer: null,
		_config: null,
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "DependencyPickerMessages");
			this.dependencyIconUrl = dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/dependency.gif');
		},
		
		postCreate: function(){
			dojo.connect(this.addButton.domNode, "onclick", this, "addSelected");
			
			this._initializeCSH();
			
			// Register the page action
			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction(vvcConstants.EDIT_DEPENDENCIES, this, "showPage");
		},

		_initializeCSH: function() {	
			this._ua = new jazz.app.ua.PageHoverHelp({uaKey: "com.ibm.team.vvc.server.core.page_ConfigurationDependencies001"});
			this._pageHeader.appendChild(this._ua.domNode);
			this._ua.domNode.style.margin = "0px 0px 0px 5px";	
	   	},
	   	
		showPage: function() {
			var uri = ContextUtils.getContextUri();
			if (uri) {
				this._config = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
				this._config.load(uri); 
			}
			this._initBreadcrumbs();
			
			if (this._pickConfigViewer) {
				this.tableContainer.domNode.removeChild(this._pickConfigViewer);
			}
			
			var params = { style: 1, lazyStart: true };
			var contextUri = ContextUtils.getContextUri();
			if (contextUri != null) {
				params.workspaceUri = contextUri; 
			}
			this._pickConfigViewer = new com.ibm.team.vvc.server.core.ui.snapshots.WorkspacesAndSnapshotsViewer(params);
			this.tableContainerNode.appendChild(this._pickConfigViewer.domNode);
			this._pickConfigViewer.load();
			this._pickConfigViewer.startup();
		},
		
		_initBreadcrumbs: function()
		{
			if (!this._breadcrumbs)
			{
				var parts = [];
				
				var explorerPageUri = PageUtil.createLink(vvcConstants.SHOW_EXPLORER);
				segment = { label: this.messages.configurations, href : explorerPageUri };
				parts.push(segment);
				
				var workspaceName = this.messages.workspace;
				if (this._config) {
					workspaceName = this._config.getTitle();
				}
				segment = { label: workspaceName, href : 'javascript:history.back()' };
				parts.push(segment);
				
				this._breadcrumbs = new jazz.ui.Breadcrumbs({segments: parts });
				
				this._breadcrumbs.placeAt(dojo.byId("breadcrumbs"));
			}
		},
		
		showMessage: function(args) {
			if (args && this._messageBoxContainer) {
				if (!this.messageBox) {
					this.messageBox = new jazz.ui.MessageBox();
					this._messageBoxContainer.appendChild(this.messageBox.domNode);
				}
				this.messageBox.showMessage(args);
				this._messageBoxContainer.style.display = "inline-block";
			}
		},
		
		hideMessage: function() {
			if (this._messageBoxContainer) {
				this._messageBoxContainer.style.display = "none";
			}
		},
		
		addSelected: function(e) {
			var okDef = new dojo.Deferred();
			InfoContentUtils.showQuestionDialog(
				this.messages.addConfirmationTitle, 
				this.messages.addConfirmation, 
				this.messages.addConfirmationOkButton, 
				this.messages.cancelButton, 
				dojo.hitch(this, function(){
					okDef.resolve(true);
				}),
				dojo.hitch(this, function(){
					okDef.reject();
				})
			);
			
			okDef.then(dojo.hitch(this, function(){;
				var selected = this._pickConfigViewer.getSelected();
				if (selected && selected.length > 0) {
					if (this.addButton) {
						this.addButton.disabled = true;
					}
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
						summary: this.messages.adding
					});
					
					var args = {};
					args.configuration = ContextUtils.getContextUri();
					args.operation = "ADD";
					args.dependencies = selected;
					uri = VvcUtils.getVvcUri() + "/" + DEPENDENCY_SERVICE;
					var self = this;
					var args = {
						url: uri,
						headers: {
							"Accept" : "application/json",
							"Content-Type" : "application/json"
						},
						sync: true,
						handleAs: "json",
						postData: dojo.toJson(args),
						error: dojo.hitch(this, function(e, ioArgs) {
							self.showMessage(PageUtil.createXhrErrorMessage(ioArgs));
						})
					};
					var deferred = jazz.client.xhrPost(args);
					deferred.addCallback(this, function (param) {
						this._waitForOperationToComplete();
						history.back();
					});
				} else {
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: this.messages.addEmptySelection
					});
				}
			}), function(){
				history.back();
			});
		},
		
		_waitForOperationToComplete: function() {
			// need to invoke the server once before return so that
			// the dependency list can be refresh properly
			var contextUri = ContextUtils.getContextUri();
			var uris = this.getConfigurationResourceUris(contextUri);
		},
		
		getConfigurationResourceUris: function(configurationUri) {
			var query = {};
			query.configurationUri = configurationUri;
			var uri = VvcUtils.getVvcUri() + "/" + DEPENDENCY_SERVICE + "?" + dojo.objectToQuery(query);
			var configResources = [];
			var args = {
					url: uri,
					headers: {
						"Accept" : "application/json"
					},
					handleAs: "json",
					sync: true,
					load: function(response, ioArgs) {
						configResources = response;
						return configResources;
					}
			};
			jazz.client.xhrGet(args);
			return configResources;
		}

	})   
})();
