/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.page._EditorWidget");

	dojo.require("jazz.ui.Breadcrumbs");
	
	dojo.require("dijit._Widget");
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.UIUtils");
	var UIUtils = com.ibm.team.vvc.shared.web.ui.util.UIUtils;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.utils.ErrorUtil");
	var ErrorUtil = com.ibm.team.vvc.shared.core.ui.utils.ErrorUtil;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory")
	var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.page._EditorWidget", [dijit._Widget], {

		messageBox: null,
		_messageBoxContainer: null,
		_resizableTables: [],
		_actionTabMap: null,
		_isPageLoaded: null,
		_resourceLink: null,

		getYPosition: function(element) {
			var yPosition = 0;
			if (element.offsetParent) {
				do {
					yPosition += element.offsetTop;
					if(!element.offsetParent)
						break;
					element = element.offsetParent;
				} while (true);
			} else if (element.y) {
				yPosition += element.y;
			}
			return yPosition;
		},
		
		addResizableChildElement: function(resizableElement) {
			if (resizableElement) {
				this._resizableTables.push(resizableElement);
				this._resizeDivNode(resizableElement);
			}
		},
		
		_resizeDivNode: function(divNode) {
			var yPosition = this.getYPosition(divNode);
			var paneHeight = document.body.offsetHeight - yPosition - 50 + "px";
			divNode.style.height = paneHeight;
		},
		
		onResize: function() {
			for (var i = 0; i < this._resizableTables.length; i++) {
				var node = this._resizableTables[i];
				if (node) {
					this._resizeDivNode(node);
				}
			}
		},
		
		showMessage: function(args) {
			UIUtils.hidePageMessage();
			if (args && this._messageBoxContainer) {
				args["closable"] = true;
				if (!this.messageBox) {
					this.messageBox = new jazz.ui.MessageBox();
					this._messageBoxContainer.appendChild(this.messageBox.domNode);
				}
				this.messageBox.showMessage(args);
				this._messageBoxContainer.style.display = "";
			}
		},
		
		hideMessage: function() {
			if (this._messageBoxContainer) {
				this._messageBoxContainer.style.display = "none";
			}
		},
		
		_getConfiguration: function(uri) {
			var configuration = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
			var link = VvcUtils.getVvcUri() + "/repository/configuration?uri=" + uri;
			var deferred = configuration.load(link);
			return configuration;
		},
		
		_handleError: function(err, args) {
			UIUtils.hidePageMessage();
			if(!this._handleCustomErrors(err)){
				ncFactory.createXhrError({xhrError: err, ioArgs: args}).dispatch();
			}
		},
		
		_handleCustomErrors: function(err) {
			UIUtils.hidePageMessage();
			var errorDetails = ErrorUtil.getErrorDetails(err);
			if (vvcConstants.ERROR_CODE_EXISTS === errorDetails.errorSubStatus) {
				this._showDuplicateNameError();
				return true;
			}
			return false;
		},
		
		_showDuplicateNameError: function() {
			ncFactory.createError({title: this.messages.duplicateConfigurationName}).dispatch();
		},
		
		_updateTab: function(tab){
			// Delay updating the hash until after the refresh method has been called.
			var hash = dojo.hash();
			if(hash){
				hash = dojo.queryToObject(hash);
			} else {
				hash = {};
			}
			
			hash.tab = tab;
			dojo.hash(dojo.objectToQuery(hash));
		},
		
		_initBreadcrumbs:function()
		{
			if (!this._breadcrumbs)
			{
				var parts = [];
				
				var segment = PageUtil.getClientAppBreadcrumbSegment();
				if (segment != null) {
					parts.push(segment);
				}
				
				var explorerPageUri = PageUtil.createLink(vvcConstants.SHOW_EXPLORER);
				segment = { label: this.messages.configurations, href : explorerPageUri };
				parts.push(segment);
				
				// if breadcrumb is specified, append it. One of the case is
				// the create snapshot action from workspace page.
				var breadcrumb = PageUtil.getRequestParmValue("breadcrumb");
				if (breadcrumb) {
					breadcrumb = decodeURIComponent(breadcrumb);
					segment = {
						label: breadcrumb, 
						href : 'javascript:history.back()'
					};
					parts.push(segment);
				}
				
				this._breadcrumbs = new jazz.ui.Breadcrumbs({segments: parts });
				
				this._breadcrumbs.placeAt(dojo.byId("breadcrumbs"));
			}
		},
	})   
})();
