/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.server.core.ui.internal.search.VvcConfigurationSearchProvider");

dojo.require("jazz.ui._SearchProvider");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcCache");
var VvcCache = com.ibm.team.vvc.shared.web.ui.util.VvcCache;

dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
var VvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.picker.ConfigurationSpacePicker");
var ConfigurationSpacePicker = com.ibm.team.vvc.shared.web.ui.picker.ConfigurationSpacePicker;

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.search", "VvcConfigurationSearchProviderMessages");
var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.search", "VvcConfigurationSearchProviderMessages");

var snapshotIcon = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/snapshot.gif');
var workspaceIcon = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/workspace.gif');

dojo.declare("com.ibm.team.vvc.server.core.ui.internal.search.VvcConfigurationSearchProvider", jazz.ui._SearchProvider, {
	
	scopes: [
		{ id: "workspaces", label: messages.searchWorkspaces, icon: workspaceIcon},
		{ id: "snapshots", label: messages.searchSnapshots, icon: snapshotIcon},
		{ id: "workspaces,snapshots", label: messages.searchAllConfigurations, icon: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/configurations.gif')},
	],
		
    defaultText: messages.defaultText,
    tooltip: messages.searchVVC,
    
    _pageSize: 10,
    
    _cache: null,
    
    _configurationSpace: null,
    _includeArchived: null,
    
    constructor: function(){
    	this._cache = new VvcCache({
    		storageKey: VvcConstants.CONFIGURATION_CACHE_KEY,
    	});
    },
        
	onUpdate: function(args){		
		var onError = dojo.hitch(this, function(error){
			// Error
			this._cache.endBulkTransaction();
			args.errback(messages.errorGettingResults);
		});
		
		dojo.when(this._cache.startBulkTransaction(), dojo.hitch(this, function(){
			dojo.when(this._getResults(args), dojo.hitch(this, function(response){
				var data = response.resultUris;
				// Build the results object
				var results = {
					results : [],
					paging: true,	// If you set this to false, the paginator will never update!
					totalResults: data.length
				};
				
				// Calculate the page to snow
				var start = (args.page - 1) * args.maxResults;
				var end = start + Math.min(args.maxResults, data.length - start);
				
				// Get the data out of the cache
				var deferredList = [];
				var resolutionIndex = 0;
				for(var i = start; i <= end; i++) {
					deferredList.push(new dojo.Deferred());
					dojo.when(this._cache.get(data[i]), dojo.hitch(this, function(conf){
						if(conf){
							results.results.push(this._createResult(conf, args));
						}
						deferredList[resolutionIndex++].resolve();
					}), onError);
				}
				
				// End the bulk transaction and wait for the items to return
				this._cache.endBulkTransaction();
				
				// Resolve the args.callback once everything is fetched from the cache
				var readyDef = new dojo.DeferredList(deferredList);
				readyDef.then(dojo.hitch(this, args.callback, results));
			}), onError);
		}), onError);
	},
	
	_getResults: function(args){
		var queryParams = {
			query: args.query,
			space: this._configurationSpace,
			showArchived: this._includeArchived,
			validTypes: args.scope
		};
		var args = {
			url: VvcUtils.getVvcUri() +"/WorkspacesQuery/query?" + dojo.objectToQuery(queryParams),
			headers: {
				"Accept" : "application/json",
				"Content-Type" : "application/json"
			},
			handleAs: "json",
			preventCache: true
		};
		return jazz.client.xhrGet(args);
	},
				
	createAdvancedOptions: function() {
		var advancedOptions = new com.ibm.team.vvc.server.core.ui.internal.search.VvcConfigurationSearchProvider._advancedProperties();
		dojo.connect(advancedOptions, "onChangeSpace", dojo.hitch(this, this._setSpaceToSearch));
		return advancedOptions.domNode;
	},
	
	_setSpaceToSearch: function(spaceUri){
		this._configurationSpace = null;
		if(spaceUri){
			this._configurationSpace = spaceUri;
		}
		this.update();
	},
	
	_createResult: function(conf, args){
		return {
			uri: conf.uri,
			icon: conf.mutable ? workspaceIcon : snapshotIcon,
			label: this._formatResultLabel(conf.title, args.query)		
		}
	},
	
	_formatResultLabel: function(string, query){
		if(!string || string.length === 0){
			return "";
		}
		
		var start = string.toLocaleLowerCase().indexOf(query.toLocaleLowerCase());
		var end = start + query.length;
		
		if(start < 0){
			return string;
		}
		
		var label = string.substring(0, start);
		label += "<b>" + string.substring(start, end) + "</b>";
		label += this._formatResultLabel(string.substring(end), query);
		
		return label;
	}
});

dojo.declare("com.ibm.team.vvc.server.core.ui.internal.search.VvcConfigurationSearchProvider._advancedProperties", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.vvc.server.core", "ui/internal/search/templates/VvcConfigurationSearchProviderAdvancedOptions.html"),
	widgetsInTemplate: false,
	_spacePicker: null,
	
	postCreate: function(){
		this._spacePicker = new ConfigurationSpacePicker();
		dojo.connect(this._spacePicker, "onSelection", dojo.hitch(this, this._onChangeSpace));
		this.domNode.appendChild(this._spacePicker.domNode);
	},
	
	_onChangeSpace: function(selectedUri){
		this.onChangeSpace(selectedUri);
	},
	
	onChangeSpace: function(selectedUri){
		// Stub
	}
	
});

})();
