/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {

	dojo.provide("com.ibm.team.vvc.server.core.ui.snapshots.ContributionsViewer");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated")
	dojo.require("jazz.ui.MessageBox");
	dojo.require('jazz.util.html');
	dojo.require('jazz.ui.StyledTable');

	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	dojo.require("jazz.ui.ResourceLink");
	var ResourceLink = jazz.ui.ResourceLink;

	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.snapshots", "SnapshotsViewerMessages");
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.snapshots", "SnapshotsViewerMessages");
	
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.snapshots", "ContributionsViewerMessages");
	var contributionsMessages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.snapshots", "ContributionsViewerMessages");
	
	var VVC_APP = "com.ibm.team.vvc.server.core";
	var CONTRIBUTIONS_SERVICE = "contributions";
	
	//style
	var WITH_ACTIONS = 1;

	dojo.declare("com.ibm.team.vvc.server.core.ui.snapshots.ContributionsViewer", [dijit._Widget, dijit._Templated], {
		messages: messages, // needed for the html template
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl(VVC_APP, "ui/snapshots/templates/WorkspacesAndSnapshotsViewer.html"),
		
		_contributionResources: null,
		_configResourcesInfo: null,
		_table: null,
		_style: null,
		_workspace: null,
		_workspaceUri: null,
		_configUris: null,
		_lazyStart: false,
		_componentNameCache: {},

		constructor: function(args) {
			if (args) {
				this._style = args.style;
				this._workspaceUri = args.workspaceUri; //consider keeping workspace parameter only
				this._workspace = args.workspace;
				this._lazyStart = args.lazyStart;
			}
		},
		
		postCreate: function() {
			if (!this._lazyStart) {
				this.load();
			}
		},

		load: function() {
			// show loading message
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: this.messages.loading
			});
			
			var deferred = this.getConfigurationResources();
			deferred.then(dojo.hitch(this, function (confResources) {
					this._contributionResources = confResources;
					this.rebuildTable(this._contributionResources);
				}),
				dojo.hitch(this, function(error) {
					this.showMessage(PageUtil.createXhrErrorMessage(deferred.ioArgs));
				})
			);
		},

		rebuildTable: function(contResources) {
			// remove loading message
			this.hideMessage();
			
			// create table
			var columnData = [];
			columnData.push({ label: messages.name, sortable: true });
			columnData.push({ label: messages.description, sortable: true });
			columnData.push({ label: contributionsMessages.mutable, sortable: true });
			columnData.push({ label: contributionsMessages.component, sortable: true });

			if (this._style === WITH_ACTIONS) {
				columnData.push({ label: messages.actions, sortable: false });
			}
			
			if (this.isShowActionBar()) {
				columnData.push({ label: messages.actions, width: "75px" });
			}
			
			if (this._table) {
				this.tableContainerNode.removeChild(this._table);
			}
			var self = this;
			var table = new jazz.ui.StyledTable({
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
				columns: columnData,
				sortBy: 1, // name field
				sortByAscending: false,
				handleSort: function() {
					this.clearBody();
					self._sort(contResources, table);
					self._fillTable(contResources, table);
				}
			});
			this._table = table;
			this.tableContainerNode.appendChild(table.domNode);
			table.domNode.style.width = "100%";

			this._fillTable(contResources, table);
		},
		
		_sort: function(configResources, table) {
			var tableColumn = this._getTableColumn;
			var f = function(a, b) {
				var c1 = tableColumn(a, table.sortBy);
				var c2 = tableColumn(b, table.sortBy);
				var r = 0;
				if (c1 && c2) {
					if (c1 < c2) {
						r = -1; 
					} else if (c1 > c2) {
						r = 1;
					}
					if (table.sortByAscending) {
						r = r * -1;
					}
				} else if (c1) {
					return -1;
				} else {
					return 1;
				}
				return r;
			};
			configResources.sort(f);
		},

		_getTableColumn: function(configResource, columnIndex) {
			var index = 0;
			
			if (columnIndex === index++) {
				return configResource.mutable;
			}
			if (columnIndex === index++) {
				return configResource.title;
			}
			if (columnIndex === index++) {
				return configResource.description;
			}
			if (columnIndex === index++) {
				return configResource.component;
			}
		},

		_fillTable: function(contribResources, table) {
			var tr, td;
			for (var i = 0; i < contribResources.length; i++) {
				
				var contribResourceUri;
				
				for (var key in contribResources[i]) {
					//there is only 1
					contribResourceUri = key;
				}
				
				tr = document.createElement("tr");
				
				td = document.createElement("td");
				
				var img = dojo.create("span", {
					"class": "contributions-viewer-contribution-icon-border"
				}, td);

				var a = dojo.create("a", {
					href: contribResourceUri,
					title: contribResources[i][contribResourceUri].name ? contribResources[i][contribResourceUri].name[0].value : "",
					innerHTML: contribResources[i][contribResourceUri].name ? contribResources[i][contribResourceUri].name[0].value : "",
				}, td);				
				
				//do not use ConfigurationUiNode or GlobalConfigurationUiNode
				//-- there is no guarantee that the resources here are on the
				//same server or are VVC based
				new ResourceLink({
					uri: a.href,
					label: a.title,
					iconNode: img
				}, a);				
				
				tr.appendChild(td);
				
				td = document.createElement("td");
				td.innerHTML = contribResources[i][contribResourceUri].description ? contribResources[i][contribResourceUri].description[0].value : "";
				tr.appendChild(td);

				td = document.createElement("td");
				
				if ("true" === contribResources[i][contribResourceUri].mutable[0].value) {
					td.innerHTML = contributionsMessages.mutableValue;
				} else if ("false" === contribResources[i][contribResourceUri].mutable[0].value) {
					td.innerHTML = contributionsMessages.immutableValue;
				} else {
					//really should not happen unless it can't be loaded
					td.innerHTML = "?";
				}
				
				tr.appendChild(td);
				
				td = document.createElement("td");
				if (this._componentNameCache[contribResourceUri] != undefined) { //empty is OK
					td.innerHTML = this._componentNameCache[contribResourceUri];
				} else {
					td.innerHTML = contribResources[i][contribResourceUri].component ? contribResources[i][contribResourceUri].component[0].value : "";
					
					var componentTitleDeferred = jazz.client.xhrGet({
						url: contribResources[i][contribResourceUri].component[0].value,
		    			headers: {
		    				"accept": "application/rdf+xml"
		    			},
		    			handleAs: "xml"
					});
				
					var context = {
						self: this,
						td: td,
						contribResourceUri: contribResourceUri
					};
					componentTitleDeferred.then(dojo.hitch(context, function(result) {
						var title = com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils.extractTitle(result);
						if (title != undefined && title != null) { //empty is OK
							this.td.innerHTML = title;
							this.self._componentNameCache[this.contribResourceUri] = title;
						}
					})); //no need for error: the url is already shown
				}
				
				tr.appendChild(td);
				
				if (this._style === WITH_ACTIONS) {
					td = document.createElement("td");
					td.appendChild(this.createActions(contribResourceUri, contribResources[i][contribResourceUri].mutable[0].value === "true").domNode);
					tr.appendChild(td);
				}

				table.tbody.appendChild(tr);
			}
			
			return table;
		},
		
		getContributionResources: function() {
			return this._contributionResources;
		},
		
		createActions: function(contribResourceUri, mutable) {
			var toolbar = new jazz.ui.Toolbar();
				
			var context = {
				contribResourceUri : contribResourceUri,
				self : this
			};
			
			var button = jazz.ui.toolbar.Button({
				title: contributionsMessages.replaceContributionAction,
				imgClass: "icons-contributions-viewer-replace",
				onClick: dojo.hitch(context, function() {
					this.self.replaceFunction(this.contribResourceUri);
				}),
				enabled: true
			});
			toolbar.addChild(button);
			
			button = jazz.ui.toolbar.Button({
				title: messages.remove,
				imgClass: "icons-contributions-viewer-delete",
				onClick: dojo.hitch(context, function() {
					this.self.deleteFunction(this.contribResourceUri);
					
				}),
				enabled: true
			});
			toolbar.addChild(button);
			
			dojo.addClass(toolbar.domNode, "vvc-workspace-snapshot-toolbar");
			

//TODO: uncomment for automatic baseline functionality
//			button = jazz.ui.toolbar.Button({
//				title: contributionsMessages.replaceWithNewStreamAutoAction,
//				imgClass: "icons-contributions-viewer-replace-with-new-stream",
//				onClick: dojo.hitch(context, function() {
//					this.self.replaceWithNewStreamAutoFunction(this.contribResourceUri);
//				}),
//				enabled: !mutable
//			});
//			toolbar.addChild(button);

//TODO: uncomment for automatic baseline functionality
//			button = jazz.ui.toolbar.Button({
//				title: contributionsMessages.replaceWithNewBaselineAutoAction,
//				imgClass: "icons-contributions-viewer-replace-with-new-baseline",
//				onClick: dojo.hitch(context, function() {
//					this.self.replaceWithNewBaselineAutoFunction(this.contribResourceUri);
//				}),
//				enabled: mutable
//			});
//			toolbar.addChild(button);
			
			return toolbar;
		},		
		
		isShowActionBar: function() {
			//TODO eventually, we will add actions
			return false;
		},
		
		showMessage: function(args) {
			if (args && this._messageBoxContainer) {
				args["closable"] = true;
				if (!this.messageBox) {
					this.messageBox = new jazz.ui.MessageBox();
					this._messageBoxContainer.appendChild(this.messageBox.domNode);
				}
				this.messageBox.showMessage(args);
				this._messageBoxContainer.style.display = "";
			}
		},
		
		hideMessage: function() {
			if (this._messageBoxContainer) {
				this._messageBoxContainer.style.display = "none";
			}
		},

		getConfigurationResources: function() {
			var uri = VvcUtils.getVvcUri();
			var args = {};
			args.configurationUri = this._workspaceUri;
			uri += ( "/" + CONTRIBUTIONS_SERVICE + "?" + dojo.objectToQuery(args) );

			var args = {
				url: uri,
				headers: {
					"Accept" : "application/json"
				},
				handleAs: "json"
			};
			
			return jazz.client.xhrGet(args);
		}

	})   
})();
